@@ -21,6 +21,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
+#include <set>
 #include <iostream>
 
 #include "Url.h"
@@ -29,6 +30,7 @@
 #include "ActionQueue.h"
 
 using std::string;
+using std::set;
 using std::cout;
 using std::endl;
 
@@ -108,7 +110,7 @@ bool ActionQueue::create(const string &database)
 bool ActionQueue::pushItem(ActionType type, const DocumentInfo &docInfo)
 {
 	string url(docInfo.getLocation());
-	string info("caption=");
+	string info(docInfo.serialize());
 	bool update = false, success = false;
 
 	// Is there already an item for this URL ?
@@ -132,40 +134,19 @@ bool ActionQueue::pushItem(ActionType type, const DocumentInfo &docInfo)
 		delete results;
 	}
 
-	// Serialize DocumentInfo
-	info += docInfo.getTitle();
-	info += "\n";
-	info += "url=";
-	info += url;
-	info += "\n";
-	info += "type=";
-	info += docInfo.getType();
-	info += "\n";
-	info += "language=";
-	info += docInfo.getLanguage();
-	info += "\n";
-	info += "modtime=";
-	info += docInfo.getTimestamp();
-	info += "\n";
-	info += "size=";
-	char sizeStr[64];
-	snprintf(sizeStr, 64, "%ld", docInfo.getSize());
-	info += sizeStr;
-	info += "\n";
-
 	if (update == false)
 	{
 		results = executeStatement("INSERT INTO ActionQueue \
 			VALUES('%q', '%q', '%q', '%d', '%q');",
 			m_queueId.c_str(), Url::escapeUrl(url).c_str(),
-			typeToText(type).c_str(), time(NULL), Url::escapeUrl(info).c_str());
+			typeToText(type).c_str(), time(NULL), info.c_str());
 	}
 	else
 	{
 		results = executeStatement("UPDATE ActionQueue \
 			SET Type='%q', Date='%d', Info='%q' WHERE \
 			QueueId='%q' AND Url='%q';",
-			typeToText(type).c_str(), time(NULL), Url::escapeUrl(info).c_str(),
+			typeToText(type).c_str(), time(NULL), info.c_str(),
 			m_queueId.c_str(), Url::escapeUrl(url).c_str());
 	}
 	if (results != NULL)
@@ -215,7 +196,7 @@ bool ActionQueue::getOldestItem(ActionType &type, DocumentInfo &docInfo)
 {
 	bool success = false;
 
-	SQLiteResults *results = executeStatement("SELECT Url, Type, Info FROM ActionQueue \
+	SQLiteResults *results = executeStatement("SELECT Type, Info FROM ActionQueue \
 		WHERE QueueId='%q' ORDER BY Date DESC LIMIT 1",
 		m_queueId.c_str());
 	if (results != NULL)
@@ -223,22 +204,11 @@ bool ActionQueue::getOldestItem(ActionType &type, DocumentInfo &docInfo)
 		SQLiteRow *row = results->nextRow();
 		if (row != NULL)
 		{
-			string url(Url::unescapeUrl(row->getColumn(0)));
-			type = textToType(row->getColumn(1));
-			string info(Url::unescapeUrl(row->getColumn(2)));
+			type = textToType(row->getColumn(0));
 			success = true;
 
 			// Deserialize DocumentInfo
-			docInfo.setTitle(StringManip::extractField(info, "caption=", "\n"));
-			docInfo.setLocation(url);
-			docInfo.setType(StringManip::extractField(info, "type=", "\n"));
-			docInfo.setLanguage(StringManip::extractField(info, "language=", "\n"));
-			docInfo.setTimestamp(StringManip::extractField(info, "modtime=", "\n"));
-			string bytesSize(StringManip::extractField(info, "size=", "\n"));
-			if (bytesSize.empty() == false)
-			{
-				docInfo.setSize((off_t )atol(bytesSize.c_str()));
-			}
+			docInfo.deserialize(row->getColumn(1));
 
 			delete row;
 		}
