@@ -71,14 +71,13 @@ bool QueryHistory::insertItem(const string &queryName, const string &engineName,
 	const string &title, const string &extract, const string &charset, float score)
 {
 	Url urlObj(url);
-	string hostName = urlObj.getHost();
-	string escapedUrl = Url::escapeUrl(url);
+	string hostName(urlObj.getHost());
 	bool success = false;
 
 	SQLiteResults *results = executeStatement("INSERT INTO QueryHistory \
 		VALUES('%q', '%q', '%q', '%q', '%q', '%q', '%q', '%f', '0.0', '%d');",
 		queryName.c_str(), engineName.c_str(), hostName.c_str(),
-		escapedUrl.c_str(), title.c_str(), extract.c_str(), charset.c_str(),
+		Url::escapeUrl(url).c_str(), title.c_str(), extract.c_str(), charset.c_str(),
 		score, time(NULL));
 	if (results != NULL)
 	{
@@ -135,6 +134,36 @@ bool QueryHistory::updateItem(const string &queryName, const string &engineName,
 	return success;
 }
 
+/// Gets the list of engines the query was run on.
+bool QueryHistory::getEngines(const string &queryName, set<string> &enginesList)
+{
+	bool success = false;
+
+	SQLiteResults *results = executeStatement("SELECT EngineName FROM QueryHistory \
+		WHERE QueryName='%q' GROUP BY EngineName",
+		queryName.c_str());
+	if (results != NULL)
+	{
+		while (results->hasMoreRows() == true)
+		{
+			SQLiteRow *row = results->nextRow();
+			if (row == NULL)
+			{
+				break;
+			}
+
+			enginesList.insert(row->getColumn(0));
+			success = true;
+
+			delete row;
+		}
+
+		delete results;
+	}
+
+	return success;
+}
+
 /// Gets the first max items for the given query, engine pair.
 bool QueryHistory::getItems(const string &queryName, const string &engineName,
 	unsigned int max, vector<DocumentInfo> &resultsList)
@@ -198,6 +227,41 @@ string QueryHistory::getItemExtract(const string &queryName, const string &engin
 	return extract;
 }
 
+/// Finds URLs.
+bool QueryHistory::findUrlsLike(const string &url, unsigned int count, set<string> &urls)
+{
+	bool success = false;
+
+	if (url.empty() == true)
+	{
+		return false; 
+	}
+
+	SQLiteResults *results = executeStatement("SELECT Url FROM QueryHistory \
+		WHERE Url LIKE '%q%%' ORDER BY Url LIMIT %u",
+		Url::escapeUrl(url).c_str(), count);
+	if (results != NULL)
+	{
+		while (results->hasMoreRows() == true)
+		{
+			SQLiteRow *row = results->nextRow();
+			if (row == NULL)
+			{
+				break;
+			}
+
+			urls.insert(Url::unescapeUrl(row->getColumn(0)));
+			success = true;
+
+			delete row;
+		}
+
+		delete results;
+	}
+
+	return success;
+}
+
 /// Gets a query's last run time.
 string QueryHistory::getLastRun(const string &queryName, const string &engineName)
 {
@@ -206,7 +270,7 @@ string QueryHistory::getLastRun(const string &queryName, const string &engineNam
 
 	if (queryName.empty() == true)
 	{
-		return lastRun;
+		return "";
 	}
 
 	if (engineName.empty() == true)
@@ -241,6 +305,23 @@ string QueryHistory::getLastRun(const string &queryName, const string &engineNam
 	return lastRun;
 }
 
+/// Deletes items at least as old as the given date.
+bool QueryHistory::deleteItems(const string &queryName, const string &engineName,
+	time_t cutOffDate)
+{
+	SQLiteResults *results = executeStatement("DELETE FROM QueryHistory \
+		WHERE QueryName='%q' AND EngineName='%q' AND Date<='%d';",
+		queryName.c_str(), engineName.c_str(), cutOffDate);
+	if (results != NULL)
+	{
+		delete results;
+
+		return true;
+	}
+
+	return false;
+}
+
 /// Deletes items.
 bool QueryHistory::deleteItems(const string &name, bool isQueryName)
 {
