@@ -21,6 +21,7 @@
 
 #include <string>
 #include <vector>
+#include <set>
 
 #include "DocumentInfo.h"
 #include "SQLiteBase.h"
@@ -52,6 +53,9 @@ class QueryHistory : public SQLiteBase
 		bool updateItem(const string &queryName, const string &engineName, const string &url,
 			const string &title, const string &extract, const string &charset, float score);
 
+		/// Gets the list of engines the query was run on.
+		bool getEngines(const string &queryName, set<string> &enginesList);
+
 		/// Gets the first max items for the given query, engine pair.
 		bool getItems(const string &queryName, const string &engineName,
 			unsigned int max, vector<DocumentInfo> &resultsList);
@@ -60,9 +64,17 @@ class QueryHistory : public SQLiteBase
 		string getItemExtract(const string &queryName, const string &engineName,
 			const string &url, string &charset);
 
+		/// Finds URLs.
+		bool findUrlsLike(const string &url, unsigned int count,
+			set<string> &urls);
+
 		/// Gets a query's last run time.
 		string getLastRun(const string &queryName, const string &engineName = "");
 
+		/// Deletes items at least as old as the given date.
+		bool deleteItems(const string &queryName, const string &engineName,
+			time_t cutOffDate);
+
 		/// Deletes items.
 		bool deleteItems(const string &name, bool isQueryName);
 
