@@ -20,10 +20,10 @@
 #define _FILTER_UTILS_H
 
 #include <string>
-#include <set>
+#include <map>
 
 #include "Document.h"
-#include "Filter.h"
+#include "filters/Filter.h"
 
 /// Utility functions for dealing with Dijon filters.
 class FilterUtils
@@ -31,6 +31,12 @@ class FilterUtils
 	public:
 		virtual ~FilterUtils();
 
+		/// Returns a Filter that handles the given MIME type, or one of its parents.
+		static Dijon::Filter *getFilter(const std::string &mimeType);
+
+		/// Indicates whether a MIME type is supported or not.
+		static bool isSupportedType(const std::string &mimeType);
+
 		/// Feeds a document's data to a filter.
 		static bool feedFilter(const Document &doc, Dijon::Filter *pFilter);
 
@@ -41,6 +47,8 @@ class FilterUtils
 		static std::string stripMarkup(const std::string &text);
 
 	protected:
+		static std::map<std::string, std::string> m_typeAliases;
+
 		FilterUtils();
 
 	private:
