@@ -29,11 +29,10 @@
 class Tokenizer
 {
 	public:
-		Tokenizer(const Document *pDocument);
+		Tokenizer(const Document *pDocument,
+			bool splitOnSpaces = false);
 		virtual ~Tokenizer();
 
-		typedef enum { ALL_DOCUMENTS = 0, ALL_BUT_FILES, NO_DOCUMENTS } DataNeeds;
-
 		/// Returns a pointer to the document being tokenized.
 		virtual const Document *getDocument(void);
 
@@ -45,6 +44,7 @@ class Tokenizer
 
 	protected:
 		const Document *m_pDocument;
+		bool m_splitOnSpaces;
 		unsigned int m_currentPos;
 
 		void setDocument(const Document *pDocument);
