@@ -19,12 +19,15 @@
 #ifndef _DBUSERVER_HH
 #define _DBUSERVER_HH
 
+#include <sys/select.h>
 #include <string>
 #include <queue>
 #include <set>
+#include <sigc++/sigc++.h>
 
 #include "MonitorInterface.h"
 #include "MonitorHandler.h"
+#include "PinotSettings.h"
 #include "WorkerThreads.h"
 
 class DaemonState : public ThreadsManager
@@ -33,25 +36,45 @@ class DaemonState : public ThreadsManager
 		DaemonState();
 		virtual ~DaemonState();
 
+		typedef enum { LOW_DISK_SPACE = 0, ON_BATTERY } StatusFlag;
+
 		void start(bool forceFullScan);
 
-		void signal_scanner(void);
+		void reload(void);
+
+		void start_crawling(void);
+
+		void stop_crawling(void);
 
 		void on_thread_end(WorkerThread *pThread);
 
 		void on_message_filefound(const DocumentInfo &docInfo, const std::string &sourceLabel,
 			bool isDirectory);
 
-		SigC::Signal1<void, int>& getQuitSignal(void);
+		sigc::signal1<void, int>& getQuitSignal(void);
+
+		void set_flag(StatusFlag flag);
+
+		bool is_flag_set(StatusFlag flag);
+
+		void reset_flag(StatusFlag flag);
 
 	protected:
 		bool m_fullScan;
+		bool m_reload;
+		fd_set m_flagsSet;
 		MonitorInterface *m_pDiskMonitor;
 		MonitorHandler *m_pDiskHandler;
-		std::string m_locationBeingCrawled;
-		SigC::Signal1<void, int> m_signalQuit;
+		sigc::connection m_timeoutConnection;
+		sigc::signal1<void, int> m_signalQuit;
+		unsigned int m_crawlers;
+		std::queue<PinotSettings::IndexableLocation> m_crawlQueue;
 
-		bool crawlLocation(const std::string &locationToCrawl, bool isSource, bool doMonitoring);
+		bool on_activity_timeout(void);
+
+		void check_battery_state(void);
+
+		bool crawl_location(const PinotSettings::IndexableLocation &location);
 
 };
 
