@@ -28,12 +28,12 @@
 #include "Url.h"
 #include "config.h"
 #include "NLS.h"
+#include "ModuleFactory.h"
 #include "PinotSettings.h"
 #include "PinotUtils.h"
 #include "EnginesTree.h"
 
 using namespace std;
-using namespace SigC;
 using namespace Glib;
 using namespace Gdk;
 using namespace Gtk;
@@ -69,17 +69,17 @@ EnginesTree::EnginesTree(VBox *enginesVbox, PinotSettings &settings) :
 	// Pack an icon renderer for engines icons
 	CellRendererPixbuf *pIconRenderer = new CellRendererPixbuf();
 	pColumn->pack_start(*manage(pIconRenderer), false);
-	pColumn->set_cell_data_func(*pIconRenderer, SigC::slot(*this, &EnginesTree::renderEngineIcon));
+	pColumn->set_cell_data_func(*pIconRenderer, sigc::mem_fun(*this, &EnginesTree::renderEngineIcon));
 	pColumn->pack_end(m_enginesColumns.m_name, false);
 	pColumn->set_sizing(TREE_VIEW_COLUMN_AUTOSIZE);
 	append_column(*manage(pColumn));
 
 	// Handle button presses
-	signal_button_press_event().connect_notify(SigC::slot(*this, &EnginesTree::onButtonPressEvent));
+	signal_button_press_event().connect_notify(sigc::mem_fun(*this, &EnginesTree::onButtonPressEvent));
 	// Control which rows can be selected
-	get_selection()->set_select_function(SigC::slot(*this, &EnginesTree::onSelectionSelect));
+	get_selection()->set_select_function(sigc::mem_fun(*this, &EnginesTree::onSelectionSelect));
 	// Listen for style changes
-	signal_style_changed().connect_notify(SigC::slot(*this, &EnginesTree::onStyleChanged));
+	signal_style_changed().connect_notify(sigc::mem_fun(*this, &EnginesTree::onStyleChanged));
 
 	// Render the icons
 	m_engineFolderIconPixbuf = render_icon(Stock::DIRECTORY, ICON_SIZE_MENU, "MetaSE-pinot");
@@ -108,15 +108,15 @@ void EnginesTree::save(void)
 		if (row[m_enginesColumns.m_type] == EnginesModelColumns::ENGINE_FOLDER)
 		{
 			ustring channelName(row[m_enginesColumns.m_name]);
-			TreeModel::Path path = m_refStore->get_path(iter);
+			TreeModel::Path channelPath = m_refStore->get_path(iter);
 
 			std::map<string, bool>::iterator channelIter = channels.find(from_utf8(channelName));
 			if (channelIter != channels.end())
 			{
 #ifdef DEBUG
-				cout << "EnginesTree::save: " << channelName << " is " << row_expanded(path) << endl;
+				cout << "EnginesTree::save: " << channelName << " is " << row_expanded(channelPath) << endl;
 #endif
-				channelIter->second = row_expanded(path);
+				channelIter->second = row_expanded(channelPath);
 			}
 		}
 	}
@@ -203,7 +203,7 @@ void EnginesTree::onButtonPressEvent(GdkEventButton *ev)
 // Handles attempts to select rows.
 //
 bool EnginesTree::onSelectionSelect(const RefPtr<TreeModel>& model,
-		const TreeModel::Path& path, bool path_currently_selected)
+		const TreeModel::Path& node_path, bool path_currently_selected)
 {
 	// All nodes can be selected
 	return true;
@@ -281,12 +281,12 @@ void EnginesTree::populate(bool indexesOnly)
 		std::set<PinotSettings::Engine>::const_iterator engineIter = engines.begin();
 		for (; engineIter != engines.end(); ++engineIter)
 		{
-			string engineName = engineIter->m_name;
-			string engineType = engineIter->m_type;
+			string engineName(engineIter->m_name);
+			string engineType(engineIter->m_type);
 
-			if (engineType == "xapian")
+			if (ModuleFactory::isSupported(engineType, true) == true)
 			{
-				// Skip Xapian, it's only usable through a local engine
+				// Skip this, it's only usable through a local engine
 				continue;
 			}
 
@@ -341,11 +341,11 @@ void EnginesTree::populate(bool indexesOnly)
 		}
 
 		TreeModel::iterator iter = m_refStore->append(localIter->children());
-		TreeModel::Row row = *iter;
-		row[m_enginesColumns.m_name] = indexName;
-		row[m_enginesColumns.m_engineName] = "xapian";
-		row[m_enginesColumns.m_option] = to_utf8(indexIter->second);
-		row[m_enginesColumns.m_type] = indexType;
+		TreeModel::Row childRow = *iter;
+		childRow[m_enginesColumns.m_name] = indexName;
+		childRow[m_enginesColumns.m_engineName] = m_settings.m_defaultBackend;
+		childRow[m_enginesColumns.m_option] = to_utf8(indexIter->second);
+		childRow[m_enginesColumns.m_type] = indexType;
 	}
 
 	// Expand this
@@ -383,7 +383,7 @@ void EnginesTree::clear(void)
 //
 // Returns the double-click signal.
 //
-Signal2<void, string, string>& EnginesTree::getDoubleClickSignal(void)
+sigc::signal2<void, string, string>& EnginesTree::getDoubleClickSignal(void)
 {
 	return m_signalDoubleClick;
 }
