@@ -29,37 +29,167 @@
 #include "StringManip.h"
 #include "Url.h"
 #include "FilterWrapper.h"
-#include "XapianDatabase.h"
-#include "FileCollector.h"
 #include "PinotSettings.h"
 #include "OnDiskHandler.h"
 
 using namespace std;
-using namespace SigC;
 
 OnDiskHandler::OnDiskHandler() :
 	MonitorHandler(),
 	m_history(PinotSettings::getInstance().getHistoryDatabaseName()),
-	m_index(PinotSettings::getInstance().m_daemonIndexLocation)
+	m_pIndex(PinotSettings::getInstance().getIndex(PinotSettings::getInstance().m_daemonIndexLocation))
 {
 	pthread_mutex_init(&m_mutex, NULL);
-	m_index.setStemmingMode(IndexInterface::STORE_BOTH);
 }
 
 OnDiskHandler::~OnDiskHandler()
 {
 	pthread_mutex_destroy(&m_mutex);
+
+	// Disconnect the signal
+	sigc::signal3<void, const DocumentInfo&, const string&, bool>::slot_list_type slotsList = m_signalFileFound.slots();
+	sigc::signal3<void, const DocumentInfo&, const string&, bool>::slot_list_type::iterator slotIter = slotsList.begin();
+	if (slotIter != slotsList.end())
+	{
+		if (slotIter->empty() == false)
+		{
+			slotIter->block();
+			slotIter->disconnect();
+		}
+	}
+
+	if (m_pIndex != NULL)
+	{
+		delete m_pIndex;
+	}
 }
 
-bool OnDiskHandler::indexFile(const string &fileName, bool alwaysUpdate, unsigned int &sourceId)
+bool OnDiskHandler::fileMoved(const string &fileName, const string &previousFileName,
+	IndexInterface::NameType type)
+{
+	set<unsigned int> docIdList;
+	bool handledEvent = false;
+
+#ifdef DEBUG
+	cout << "OnDiskHandler::fileMoved: " << fileName << endl;
+#endif
+	if (m_pIndex == NULL)
+	{
+		return false;
+	}
+
+	pthread_mutex_lock(&m_mutex);
+	// Get a list of documents in that directory/file
+	if (type == IndexInterface::BY_FILE)
+	{
+		m_pIndex->listDocuments(string("file://") + previousFileName, docIdList, type);
+	}
+	else
+	{
+		m_pIndex->listDocuments(previousFileName, docIdList, type);
+	}
+	// ...and the directory/file itself
+	unsigned int baseDocId = m_pIndex->hasDocument(string("file://") + previousFileName);
+	if (baseDocId > 0)
+	{
+		docIdList.insert(baseDocId);
+	}
+	if (docIdList.empty() == false)
+	{
+		for (set<unsigned int>::const_iterator iter = docIdList.begin();
+			iter != docIdList.end(); ++iter)
+		{
+			DocumentInfo docInfo;
+
+#ifdef DEBUG
+			cout << "OnDiskHandler::fileMoved: moving " << *iter << endl;
+#endif
+			if (m_pIndex->getDocumentInfo(*iter, docInfo) == true)
+			{
+				string newLocation(docInfo.getLocation());
+
+				if (baseDocId == *iter)
+				{
+					Url previousUrlObj(previousFileName), urlObj(fileName);
+
+					// Update the title if it was the directory/file name
+					if (docInfo.getTitle() == previousUrlObj.getFile())
+					{
+						docInfo.setTitle(urlObj.getFile());
+					}
+				}
+
+				string::size_type pos = newLocation.find(previousFileName);
+				if (pos != string::npos)
+				{
+					newLocation.replace(pos, previousFileName.length(), fileName);
+
+					// Change the location
+					docInfo.setLocation(newLocation);
+
+					handledEvent = replaceFile(*iter, docInfo);
+#ifdef DEBUG
+					cout << "OnDiskHandler::fileMoved: moved " << *iter << ", " << docInfo.getLocation() << endl;
+#endif
+				}
+#ifdef DEBUG
+				else cout << "OnDiskHandler::fileMoved: skipping " << newLocation << endl;
+#endif
+			}
+		}
+	}
+#ifdef DEBUG
+	else cout << "OnDiskHandler::fileMoved: no documents in " << previousFileName << endl;
+#endif
+	pthread_mutex_unlock(&m_mutex);
+
+	return handledEvent;
+}
+
+bool OnDiskHandler::fileDeleted(const string &fileName, IndexInterface::NameType type)
+{
+	set<unsigned int> docIdList;
+	string location(string("file://") + fileName);
+	bool unindexedDocs = false, handledEvent = false;
+
+#ifdef DEBUG
+	cout << "OnDiskHandler::fileDeleted: " << fileName << endl;
+#endif
+	if (m_pIndex == NULL)
+	{
+		return false;
+	}
+
+	pthread_mutex_lock(&m_mutex);
+	// Unindex all of the directory/file's documents
+	if (type == IndexInterface::BY_FILE)
+	{
+		unindexedDocs = m_pIndex->unindexDocuments(location, type);
+	}
+	else
+	{
+		unindexedDocs = m_pIndex->unindexDocuments(fileName, type);
+	}
+	if (unindexedDocs == true)
+	{
+		// ...as well as the actual directory/file
+		m_pIndex->unindexDocument(location);
+
+		m_history.deleteItems(location);
+		handledEvent = true;
+	}
+	pthread_mutex_unlock(&m_mutex);
+
+	return handledEvent;
+}
+
+bool OnDiskHandler::indexFile(const string &fileName, bool isDirectory, unsigned int &sourceId)
 {
 	string location(string("file://") + fileName);
 	Url urlObj(location);
-	set<string> labels;
-	bool indexedFile = false;
+	char labelStr[64];
 
-	if ((m_index.isGood() == false) || 
-		(fileName.empty() == true))
+	if (fileName.empty() == true)
 	{
 		return false;
 	}
@@ -70,15 +200,6 @@ bool OnDiskHandler::indexFile(const string &fileName, bool alwaysUpdate, unsigne
 		return false;
 	}
 
-	// Has this file been indexed already ?
-	unsigned int docId = m_index.hasDocument(location);
-	if ((docId > 0) &&
-		(alwaysUpdate == false))
-	{
-		// No need to update
-		return true;
-	}
-
 	// What source does it belong to ?
 	for(map<unsigned int, string>::const_iterator sourceIter = m_fileSources.begin();
 		sourceIter != m_fileSources.end(); ++sourceIter)
@@ -93,63 +214,33 @@ bool OnDiskHandler::indexFile(const string &fileName, bool alwaysUpdate, unsigne
 
 		if (sourceIter->second.substr(0, location.length()) == location)
 		{
-			char sourceStr[64];
 
 			// That's the one
-			snprintf(sourceStr, 64, "SOURCE%u", sourceIter->first);
-			labels.insert(sourceStr);
+			snprintf(labelStr, 64, "X-SOURCE%u", sourceIter->first);
 			break;
 		}
 	}
 
-	DocumentInfo docInfo(urlObj.getFile(), location, MIMEScanner::scanUrl(urlObj), "");
+	DocumentInfo docInfo("", location, MIMEScanner::scanUrl(urlObj), "");
 
-	FileCollector fileCollector;
-	Document *pDoc = fileCollector.retrieveUrl(docInfo);
-	if (pDoc == NULL)
-	{
-#ifdef DEBUG
-		cout << "OnDiskHandler::indexFile: couldn't download " << location << endl;
-#endif
+	m_signalFileFound(docInfo, labelStr, isDirectory);
 
-		// The file  couldn't be downloaded but exists nonetheless !
-		pDoc = new Document(docInfo);
-	}
-
-	FilterWrapper wrapFilter(&m_index);
-
-	// Get an ad hoc tokenizer for the message
-	// Index or update ?
-	if (docId == 0)
-	{
-		indexedFile = wrapFilter.indexDocument(*pDoc, labels, docId);
-	}
-	else
-	{
-		indexedFile = wrapFilter.updateDocument(*pDoc, docId);
-	}
-
-#ifdef DEBUG
-	if (indexedFile == false)
-	{
-		cout << "OnDiskHandler::indexFile: couldn't index " << location << endl;
-	}
-#endif
-
-	delete pDoc;
-
-	return indexedFile;
+	return true;
 }
 
 bool OnDiskHandler::replaceFile(unsigned int docId, DocumentInfo &docInfo)
 {
-	FilterWrapper wrapFilter(&m_index);
+	if (m_pIndex == NULL)
+	{
+		return false;
+	}
 
 	// Unindex the destination file
+	FilterWrapper wrapFilter(m_pIndex);
 	wrapFilter.unindexDocument(docInfo.getLocation());
 
 	// Update the document info
-	return m_index.updateDocumentInfo(docId, docInfo);
+	return m_pIndex->updateDocumentInfo(docId, docInfo);
 }
 
 void OnDiskHandler::initialize(void)
@@ -181,16 +272,17 @@ void OnDiskHandler::initialize(void)
 			// Is this an indexable location ?
 			if (directories.find(sourceIter->second.substr(7)) == directories.end())
 			{
-				char sourceStr[64];
+				char labelStr[64];
 
-				snprintf(sourceStr, 64, "SOURCE%u", sourceId);
+				snprintf(labelStr, 64, "X-SOURCE%u", sourceId);
 
 #ifdef DEBUG
 				cout << "OnDiskHandler::initialize: " << sourceIter->second
 					<< ", source " << sourceId << " was removed" << endl;
 #endif
 				// All documents with this label will be unindexed
-				if (m_index.unindexDocuments(sourceStr, IndexInterface::BY_LABEL) == true)
+				if ((m_pIndex != NULL) &&
+					(m_pIndex->unindexDocuments(labelStr, IndexInterface::BY_LABEL) == true))
 				{
 					// Delete the source itself and all its items
 					m_history.deleteSource(sourceId);
@@ -208,7 +300,10 @@ void OnDiskHandler::initialize(void)
 void OnDiskHandler::flushIndex(void)
 {
 	pthread_mutex_lock(&m_mutex);
-	m_index.flush();
+	if (m_pIndex != NULL)
+	{
+		m_pIndex->flush();
+	}
 	pthread_mutex_unlock(&m_mutex);
 }
 
@@ -227,17 +322,45 @@ bool OnDiskHandler::fileCreated(const string &fileName)
 	cout << "OnDiskHandler::fileCreated: " << fileName << endl;
 #endif
 	pthread_mutex_lock(&m_mutex);
-	// The file shouldn't exist in the index, but if it does for some reason, don't update it
+	// The file may exist in the index
 	handledEvent = indexFile(fileName, false, sourceId);
 	if (handledEvent == true)
 	{
-		m_history.insertItem("file://" + fileName, CrawlHistory::CRAWLED, sourceId, time(NULL));
+	        string location("file://" + fileName);
+		CrawlHistory::CrawlStatus status = CrawlHistory::UNKNOWN;
+	        time_t itemDate;
+
+		// ...and therefore may exist in the history database
+		if (m_history.hasItem(location, status, itemDate) == true)
+		{
+			m_history.updateItem(location, CrawlHistory::CRAWLED, time(NULL));
+		}
+		else
+		{
+			m_history.insertItem(location, CrawlHistory::CRAWLED, sourceId, time(NULL));
+		}
 	}
 	pthread_mutex_unlock(&m_mutex);
 
 	return handledEvent;
 }
 
+bool OnDiskHandler::directoryCreated(const string &dirName)
+{
+	unsigned int sourceId;
+	bool handledEvent = false;
+
+#ifdef DEBUG
+	cout << "OnDiskHandler::directoryCreated: " << dirName << endl;
+#endif
+	pthread_mutex_lock(&m_mutex);
+	handledEvent = indexFile(dirName, true, sourceId);
+	// History will be set by crawling
+	pthread_mutex_unlock(&m_mutex);
+
+	return handledEvent;
+}
+
 bool OnDiskHandler::fileModified(const string &fileName)
 {
 	unsigned int sourceId;
@@ -248,7 +371,7 @@ bool OnDiskHandler::fileModified(const string &fileName)
 #endif
 	pthread_mutex_lock(&m_mutex);
 	// Update the file, or index if necessary
-	handledEvent = indexFile(fileName, true, sourceId);
+	handledEvent = indexFile(fileName, false, sourceId);
 	if (handledEvent == true)
 	{
 		m_history.updateItem("file://" + fileName, CrawlHistory::CRAWLED, time(NULL));
@@ -260,116 +383,27 @@ bool OnDiskHandler::fileModified(const string &fileName)
 
 bool OnDiskHandler::fileMoved(const string &fileName, const string &previousFileName)
 {
-	bool handledEvent = false;
-
-#ifdef DEBUG
-	cout << "OnDiskHandler::fileMoved: " << fileName << endl;
-#endif
-	pthread_mutex_lock(&m_mutex);
-	unsigned int oldDocId = m_index.hasDocument(string("file://") + previousFileName);
-	if (oldDocId > 0)
-	{
-		DocumentInfo docInfo;
-
-		if (m_index.getDocumentInfo(oldDocId, docInfo) == true)
-		{
-			// Change the location
-			docInfo.setLocation(string("file://") + fileName);
-
-			handledEvent = replaceFile(oldDocId, docInfo);
-		}
-	}
-	pthread_mutex_unlock(&m_mutex);
-
-	return handledEvent;
+	return fileMoved(fileName, previousFileName, IndexInterface::BY_FILE);
 }
 
 bool OnDiskHandler::directoryMoved(const string &dirName,
 	const string &previousDirName)
 {
-	set<unsigned int> docIdList;
-	bool handledEvent = false;
-
-#ifdef DEBUG
-	cout << "OnDiskHandler::directoryMoved: " << dirName << endl;
-#endif
-	pthread_mutex_lock(&m_mutex);
-	if (m_index.listDocumentsInDirectory(previousDirName, docIdList) == true)
-	{
-		for (set<unsigned int>::const_iterator iter = docIdList.begin();
-			iter != docIdList.end(); ++iter)
-		{
-			DocumentInfo docInfo;
-
-			if (m_index.getDocumentInfo(*iter, docInfo) == true)
-			{
-				string newLocation(docInfo.getLocation());
-
-				string::size_type pos = newLocation.find(previousDirName);
-				if (pos != string::npos)
-				{
-					newLocation.replace(pos, previousDirName.length(), dirName);
-
-					// Change the location
-					docInfo.setLocation(newLocation);
-
-					replaceFile(*iter, docInfo);
-				}
-			}
-		}
-
-		handledEvent = true;
-	}
-#ifdef DEBUG
-	else cout << "OnDiskHandler::directoryMoved: no documents in " << previousDirName << endl;
-#endif
-	pthread_mutex_unlock(&m_mutex);
-
-	return handledEvent;
+	return fileMoved(dirName, previousDirName, IndexInterface::BY_DIRECTORY);
 }
 
 bool OnDiskHandler::fileDeleted(const string &fileName)
 {
-	FilterWrapper wrapFilter(&m_index);
-	string location("file://");
-	bool handledEvent = true;
-
-#ifdef DEBUG
-	cout << "OnDiskHandler::fileDeleted: " << fileName << endl;
-#endif
-	location += fileName;
-	pthread_mutex_lock(&m_mutex);
-	unsigned int docId = m_index.hasDocument(location);
-	if (docId > 0)
-	{
-		// Unindex the file
-		handledEvent = wrapFilter.unindexDocument(location);
-		if (handledEvent == true)
-		{
-			m_history.deleteItem(location);
-		}
-	}
-	pthread_mutex_unlock(&m_mutex);
-
-	return handledEvent;
+	return fileDeleted(fileName, IndexInterface::BY_FILE);
 }
 
 bool OnDiskHandler::directoryDeleted(const string &dirName)
 {
-	set<unsigned int> docIdList;
-	bool handledEvent = false;
+	return fileDeleted(dirName, IndexInterface::BY_DIRECTORY);
+}
 
-#ifdef DEBUG
-	cout << "OnDiskHandler::directoryDeleted: " << dirName << endl;
-#endif
-	pthread_mutex_lock(&m_mutex);
-	if (m_index.unindexDocuments(dirName, IndexInterface::BY_DIRECTORY) == true)
-	{
-		m_history.deleteItems(string("file://") + dirName);
-		handledEvent = true;
-	}
-	pthread_mutex_unlock(&m_mutex);
-
-	return handledEvent;
+sigc::signal3<void, const DocumentInfo&, const string&, bool>& OnDiskHandler::getFileFoundSignal(void)
+{
+	return m_signalFileFound;
 }
 
