@@ -23,11 +23,11 @@
 #include <pthread.h>
 #include <string>
 #include <map>
-#include <sigc++/slot.h>
+#include <sigc++/sigc++.h>
 
 #include "CrawlHistory.h"
+#include "IndexInterface.h"
 #include "MonitorHandler.h"
-#include "XapianIndex.h"
 #include "PinotSettings.h"
 
 class OnDiskHandler : public MonitorHandler
@@ -48,6 +48,9 @@ class OnDiskHandler : public MonitorHandler
 		/// Handles file creation events.
 		virtual bool fileCreated(const std::string &fileName);
 
+		/// Handles directory creation events.
+		virtual bool directoryCreated(const std::string &dirName);
+
 		/// Handles file modified events.
 		virtual bool fileModified(const std::string &fileName);
 
@@ -65,13 +68,22 @@ class OnDiskHandler : public MonitorHandler
 		/// Handles directory deleted events.
 		virtual bool directoryDeleted(const std::string &dirName);
 
+		sigc::signal3<void, const DocumentInfo&, const std::string&, bool>& getFileFoundSignal(void);
+
 	protected:
 		pthread_mutex_t m_mutex;
+		sigc::signal3<void, const DocumentInfo&, const std::string&, bool> m_signalFileFound;
 		std::map<unsigned int, std::string> m_fileSources;
 		CrawlHistory m_history;
-		XapianIndex m_index;
+		IndexInterface *m_pIndex;
 
-		bool indexFile(const std::string &fileName, bool alwaysUpdate, unsigned int &sourceId);
+		bool fileMoved(const std::string &fileName,
+			const std::string &previousFileName,
+			IndexInterface::NameType type);
+
+		bool fileDeleted(const std::string &fileName, IndexInterface::NameType type);
+
+		bool indexFile(const std::string &fileName, bool isDirectory, unsigned int &sourceId);
 
 		bool replaceFile(unsigned int docId, DocumentInfo &docInfo);
 
