@@ -34,11 +34,11 @@
 #include <libxml++/nodes/textnode.h>
 
 #include "config.h"
+#include "NLS.h"
 #include "CommandLine.h"
 #include "Languages.h"
-#include "NLS.h"
 #include "StringManip.h"
-#include "IndexFactory.h"
+#include "ModuleFactory.h"
 #include "PluginWebEngine.h"
 #include "PinotSettings.h"
 
@@ -90,6 +90,9 @@ PinotSettings PinotSettings::m_instance;
 bool PinotSettings::m_clientMode = false;
 
 PinotSettings::PinotSettings() :
+	m_warnAboutVersion(false),
+	m_defaultBackend("xapian"),
+	m_minimumDiskSpace(50),
 	m_xPos(0),
 	m_yPos(0),
 	m_width(0),
@@ -130,6 +133,14 @@ PinotSettings::PinotSettings() :
 	m_docsIndexLocation += "/index";
 	m_daemonIndexLocation = directoryName;
 	m_daemonIndexLocation += "/daemon";
+
+	// This is not set in the configuration file
+	char *minDiskSpace = getenv("PINOT_MINIMUM_DISK_SPACE");
+	if ((minDiskSpace != NULL) &&
+		(strlen(minDiskSpace) > 0))
+	{
+		m_minimumDiskSpace = atof(minDiskSpace);
+	}
 }
 
 PinotSettings::~PinotSettings()
@@ -150,28 +161,35 @@ bool PinotSettings::enableClientMode(bool enable)
 	return isEnabled;
 }
 
-string PinotSettings::getConfigurationDirectory(void)
+string PinotSettings::getHomeDirectory(void)
 {
 	struct passwd *pPasswd = getpwuid(geteuid());
-	string directoryName;
 
 	if ((pPasswd != NULL) &&
 		(pPasswd->pw_dir != NULL))
 	{
-		directoryName = pPasswd->pw_dir;
+		return pPasswd->pw_dir;
 	}
 	else
 	{
 		char *homeDir = getenv("HOME");
 		if (homeDir != NULL)
 		{
-			directoryName = homeDir;
-		}
-		else
-		{
-			return "~/.pinot";
+			return homeDir;
 		}
 	}
+
+	return "~";
+}
+
+string PinotSettings::getConfigurationDirectory(void)
+{
+	string directoryName(getHomeDirectory());
+
+	if (directoryName.empty() == true)
+	{
+		return "~/.pinot";
+	}
 	directoryName += "/.pinot";
 
 	return directoryName;
@@ -246,16 +264,40 @@ bool PinotSettings::isFirstRun(void) const
 
 void PinotSettings::clear(void)
 {
-	// Clear lists
+	m_version.clear();
+	m_warnAboutVersion = false;
+	m_defaultBackend = "xapian";
+	m_googleAPIKey.clear();
+	m_xPos = 0;
+	m_yPos = 0;
+	m_width = 0;
+	m_height = 0;
+	m_panePos = -1;
+	m_showEngines = false;
+	m_expandQueries = false;
+	m_ignoreRobotsDirectives = false;
+	m_suggestQueryTerms = true;
+	m_newResultsColourRed = 65535;
+	m_newResultsColourGreen = 0;
+	m_newResultsColourBlue = 0;
+	m_proxyAddress.clear();
+	m_proxyPort = 8080;
+	m_proxyType.clear();
+	m_proxyEnabled = false;
+	m_indexableLocations.clear();
+	m_filePatternsList.clear();
+	m_isBlackList = false;
+	m_cacheProviders.clear();
+	m_cacheProtocols.clear();
+
+	m_firstRun = false;
 	m_indexNames.clear();
 	m_indexIds.clear();
 	m_engines.clear();
 	m_engineIds.clear();
+	m_engineChannels.clear();
 	m_queries.clear();
 	m_labels.clear();
-	m_indexableLocations.clear();
-	m_filePatternsList.clear();
-	m_cacheProviders.clear();
 }
 
 bool PinotSettings::loadGlobal(const string &fileName)
@@ -289,25 +331,55 @@ bool PinotSettings::load(void)
 		m_labels.insert(_("New"));
 		m_labels.insert(_("Personal"));
 		// Skip common image, video and archive types
+		m_filePatternsList.insert("*~");
 		m_filePatternsList.insert("*.Z");
+		m_filePatternsList.insert("*.a");
 		m_filePatternsList.insert("*.avi");
 		m_filePatternsList.insert("*.asf");
+		m_filePatternsList.insert("*.cap");
 		m_filePatternsList.insert("*.gif");
 		m_filePatternsList.insert("*.iso");
 		m_filePatternsList.insert("*.jpeg");
 		m_filePatternsList.insert("*.jpg");
+		m_filePatternsList.insert("*.la");
 		m_filePatternsList.insert("*.lha");
 		m_filePatternsList.insert("*.mov");
 		m_filePatternsList.insert("*.msf");
 		m_filePatternsList.insert("*.mpeg");
 		m_filePatternsList.insert("*.mpg");
+		m_filePatternsList.insert("*.o");
 		m_filePatternsList.insert("*.png");
 		m_filePatternsList.insert("*.rar");
 		m_filePatternsList.insert("*.sh");
+		m_filePatternsList.insert("*.so");
 		m_filePatternsList.insert("*.tiff");
 		m_filePatternsList.insert("*.wmv");
 		m_filePatternsList.insert("*.xbm");
 		m_filePatternsList.insert("*.xpm");
+		// Create default queries
+		struct passwd *pPasswd = getpwuid(geteuid());
+		if (pPasswd != NULL)
+		{
+			string userName;
+
+			if ((pPasswd->pw_gecos != NULL) &&
+				(strlen(pPasswd->pw_gecos) > 0))
+			{
+				userName = pPasswd->pw_gecos;
+			}
+			else if (pPasswd->pw_name != NULL)
+			{
+				userName = pPasswd->pw_name;
+			}
+
+			if (userName.empty() == false)
+			{
+				QueryProperties queryProps(_("Me"), userName);
+
+				queryProps.setSortOrder(QueryProperties::DATE);
+				addQuery(queryProps);
+			}
+		}
 	}
 
 	// Some search engines are hardcoded
@@ -316,8 +388,18 @@ bool PinotSettings::load(void)
 	m_engines.insert(Engine("Google API", "googleapi", "", "The Web"));
 	m_engineChannels.insert(pair<string, bool>("The Web", true));
 #endif
-	m_engineIds[1 << m_engines.size()] = "Xapian";
-	m_engines.insert(Engine("Xapian", "xapian", "", ""));
+	// Others are available as back-ends
+	map<string, bool> engines;
+	ModuleFactory::getSupportedEngines(engines);
+	for (map<string, bool>::const_iterator engineIter = engines.begin();
+		engineIter != engines.end(); ++engineIter)
+	{
+		if (engineIter->second == true)
+		{
+			m_engineIds[1 << m_engines.size()] = engineIter->first;
+			m_engines.insert(Engine(engineIter->first, engineIter->first, "", ""));
+		}
+	}
 
 	return true;
 }
@@ -392,6 +474,21 @@ bool PinotSettings::loadConfiguration(const std::string &fileName, bool isGlobal
 				{
 					m_version = nodeContent;
 				}
+				else if (nodeName == "warnaboutversion")
+				{
+					if (nodeContent == "YES")
+					{
+						m_warnAboutVersion = true;
+					}
+					else
+					{
+						m_warnAboutVersion = false;
+					}
+				}
+				else if (nodeName == "backend")
+				{
+					m_defaultBackend = nodeContent;
+				}
 				else if (nodeName == "googleapikey")
 				{
 					m_googleAPIKey = nodeContent;
@@ -484,14 +581,14 @@ bool PinotSettings::loadUi(const Element *pElem)
 	for (Node::NodeList::iterator iter = childNodes.begin(); iter != childNodes.end(); ++iter)
 	{
 		Node *pNode = (*iter);
-		Element *pElem = dynamic_cast<Element*>(pNode);
-		if (pElem == NULL)
+		Element *pChildElem = dynamic_cast<Element*>(pNode);
+		if (pChildElem == NULL)
 		{
 			continue;
 		}
 
-		string nodeName = pElem->get_name();
-		string nodeContent = getElementContent(pElem);
+		string nodeName(pChildElem->get_name());
+		string nodeContent(getElementContent(pChildElem));
 		if (nodeName == "xpos")
 		{
 			m_xPos = atoi(nodeContent.c_str());
@@ -557,14 +654,14 @@ bool PinotSettings::loadIndexes(const Element *pElem)
 	for (Node::NodeList::iterator iter = childNodes.begin(); iter != childNodes.end(); ++iter)
 	{
 		Node *pNode = (*iter);
-		Element *pElem = dynamic_cast<Element*>(pNode);
-		if (pElem == NULL)
+		Element *pChildElem = dynamic_cast<Element*>(pNode);
+		if (pChildElem == NULL)
 		{
 			continue;
 		}
 
-		string nodeName = pElem->get_name();
-		string nodeContent = getElementContent(pElem);
+		string nodeName(pChildElem->get_name());
+		string nodeContent(getElementContent(pChildElem));
 		if (nodeName == "name")
 		{
 			indexName = nodeContent;
@@ -600,14 +697,14 @@ bool PinotSettings::loadEngineChannels(const Element *pElem)
 	for (Node::NodeList::iterator iter = childNodes.begin(); iter != childNodes.end(); ++iter)
 	{
 		Node *pNode = (*iter);
-		Element *pElem = dynamic_cast<Element*>(pNode);
-		if (pElem == NULL)
+		Element *pChildElem = dynamic_cast<Element*>(pNode);
+		if (pChildElem == NULL)
 		{
 			continue;
 		}
 
-		string nodeName = pElem->get_name();
-		string nodeContent = getElementContent(pElem);
+		string nodeName(pChildElem->get_name());
+		string nodeContent(getElementContent(pChildElem));
 		if (nodeName == "name")
 		{
 			std::map<string, bool>::iterator channelIter = m_engineChannels.find(nodeContent);
@@ -637,112 +734,125 @@ bool PinotSettings::loadQueries(const Element *pElem)
 
 	QueryProperties queryProps;
 	Date minDate, maxDate;
-	string backCompatString;
+	string freeQuery;
 	bool enableMinDate = false, enableMaxDate = false;
 
-	minDate.set_time(time(NULL));
-	maxDate.set_time(time(NULL));
-
 	// Load the query's properties
 	for (Node::NodeList::iterator iter = childNodes.begin(); iter != childNodes.end(); ++iter)
 	{
 		Node *pNode = (*iter);
-		Element *pElem = dynamic_cast<Element*>(pNode);
-		if (pElem == NULL)
+		Element *pChildElem = dynamic_cast<Element*>(pNode);
+		if (pChildElem == NULL)
 		{
 			continue;
 		}
 
-		string nodeName = pElem->get_name();
-		string nodeContent = getElementContent(pElem);
+		string nodeName(pChildElem->get_name());
+		string nodeContent(getElementContent(pChildElem));
 		if (nodeName == "name")
 		{
 			queryProps.setName(nodeContent);
 		}
+		else if (nodeName == "sortorder")
+		{
+			if (nodeContent == "DATE")
+			{
+				queryProps.setSortOrder(QueryProperties::DATE);
+			}
+			else
+			{
+				queryProps.setSortOrder(QueryProperties::RELEVANCE);
+			}
+		}
 		else if (nodeName == "text")
 		{
-			queryProps.setFreeQuery(nodeContent);
+			freeQuery = nodeContent;
 		}
 		else if ((nodeName == "and") &&
 			(nodeContent.empty() == false))
 		{
-			if (backCompatString.empty() == false)
+			if (freeQuery.empty() == false)
 			{
-				backCompatString += " ";
+				freeQuery += " ";
 			}
-			backCompatString += nodeContent;
+			freeQuery += nodeContent;
 		}
 		else if ((nodeName == "phrase") &&
 			(nodeContent.empty() == false))
 		{
-			if (backCompatString.empty() == false)
+			if (freeQuery.empty() == false)
 			{
-				backCompatString += " ";
+				freeQuery += " ";
 			}
-			backCompatString += "\"";
-			backCompatString += nodeContent;
-			backCompatString += "\"";
+			freeQuery += "\"";
+			freeQuery += nodeContent;
+			freeQuery += "\"";
 		}
 		else if ((nodeName == "any") &&
 			(nodeContent.empty() == false))
 		{
 			// FIXME: don't be lazy and add those correctly
-			if (backCompatString.empty() == false)
+			if (freeQuery.empty() == false)
 			{
-				backCompatString += " ";
+				freeQuery += " ";
 			}
-			backCompatString += nodeContent;
+			freeQuery += nodeContent;
 		}
 		else if ((nodeName == "not") &&
 			(nodeContent.empty() == false))
 		{
-			if (backCompatString.empty() == false)
+			if (freeQuery.empty() == false)
 			{
-				backCompatString += " ";
+				freeQuery += " ";
 			}
-			backCompatString += "-(";
-			backCompatString += nodeContent;
-			backCompatString += ")";
+			freeQuery += "-(";
+			freeQuery += nodeContent;
+			freeQuery += ")";
 		}
 		else if ((nodeName == "language") &&
 			(nodeContent.empty() == false))
 		{
-			if (backCompatString.empty() == false)
+			if (freeQuery.empty() == false)
 			{
-				backCompatString += " ";
+				freeQuery += " ";
 			}
-			backCompatString += "language:";
-			backCompatString += nodeContent;
+			freeQuery += "lang:";
+			freeQuery += nodeContent;
+		}
+		else if ((nodeName == "stemlanguage") &&
+			(nodeContent.empty() == false))
+		{
+			queryProps.setStemmingLanguage(Languages::toLocale(nodeContent));
 		}
 		else if ((nodeName == "hostfilter") &&
 			(nodeContent.empty() == false))
 		{
-			if (backCompatString.empty() == false)
+			if (freeQuery.empty() == false)
 			{
-				backCompatString += " ";
+				freeQuery += " ";
 			}
-			backCompatString += "site:";
-			backCompatString += nodeContent;
+			freeQuery += "site:";
+			freeQuery += nodeContent;
 		}
 		else if ((nodeName == "filefilter") &&
 			(nodeContent.empty() == false))
 		{
-			if (backCompatString.empty() == false)
+			if (freeQuery.empty() == false)
 			{
-				backCompatString += " ";
+				freeQuery += " ";
 			}
-			backCompatString += "file:";
-			backCompatString += nodeContent;
+			freeQuery += "file:";
+			freeQuery += nodeContent;
 		}
 		else if ((nodeName == "labelfilter") &&
 			(nodeContent.empty() == false))
 		{
-			if (backCompatString.empty() == false)
+			if (freeQuery.empty() == false)
 			{
-				backCompatString += " ";
+				freeQuery += " ";
 			}
-			backCompatString += "label:";
-			backCompatString += nodeContent;
+			freeQuery += "label:";
+			freeQuery += nodeContent;
 		}
 		else if (nodeName == "maxresults")
 		{
@@ -755,12 +865,10 @@ bool PinotSettings::loadQueries(const Element *pElem)
 			{
 				enableMinDate = true;
 			}
-			queryProps.setMinimumDate(enableMinDate, minDate.get_day(), minDate.get_month(), minDate.get_year());
 		}
 		else if (nodeName == "mindate")
 		{
 			minDate.set_parse(nodeContent);
-			queryProps.setMinimumDate(enableMinDate, minDate.get_day(), minDate.get_month(), minDate.get_year());
 		}
 		else if (nodeName == "enablemaxdate")
 		{
@@ -768,12 +876,10 @@ bool PinotSettings::loadQueries(const Element *pElem)
 			{
 				enableMaxDate = true;
 			}
-			queryProps.setMaximumDate(enableMaxDate, maxDate.get_day(), maxDate.get_month(), maxDate.get_year());
 		}
 		else if (nodeName == "maxdate")
 		{
 			maxDate.set_parse(nodeContent);
-			queryProps.setMaximumDate(enableMaxDate, maxDate.get_day(), maxDate.get_month(), maxDate.get_year());
 		}
 		else if (nodeName == "index")
 		{
@@ -790,14 +896,51 @@ bool PinotSettings::loadQueries(const Element *pElem)
 		{
 			queryProps.setLabelName(nodeContent);
 		}
+		else if (nodeName == "modified")
+		{
+			if (nodeContent == "YES")
+			{
+				queryProps.setModified(true);
+			}
+		}
+	}
+
+	// Are pre-0.80 dates specified ?
+	if ((enableMinDate == true) ||
+		(enableMaxDate == true))
+	{
+		// Provide reasonable defaults
+		if (enableMinDate == false)
+		{
+			minDate.set_day(1);
+			minDate.set_month(Date::JANUARY);
+			minDate.set_year(1970);
+		}
+		if (enableMaxDate == false)
+		{
+			maxDate.set_day(31);
+			maxDate.set_month(Date::DECEMBER);
+			maxDate.set_year(2099);
+		}
+
+		ustring minDateStr(minDate.format_string("%Y%m%d"));
+		ustring maxDateStr(maxDate.format_string("%Y%m%d"));
+
+#ifdef DEBUG
+		cout << "PinotSettings::loadQueries: date range " << minDateStr << " to " << maxDateStr << endl;
+#endif
+		freeQuery += " ";
+		freeQuery += minDateStr;
+		freeQuery += "..";
+		freeQuery += maxDateStr;
 	}
 
 	// We need at least a name
 	if (queryProps.getName().empty() == false)
 	{
-		if (backCompatString.empty() == false)
+		if (freeQuery.empty() == false)
 		{
-			queryProps.setFreeQuery(backCompatString);
+			queryProps.setFreeQuery(freeQuery);
 		}
 		m_queries[queryProps.getName()] = queryProps;
 	}
@@ -822,14 +965,14 @@ bool PinotSettings::loadLabels(const Element *pElem)
 	for (Node::NodeList::iterator iter = childNodes.begin(); iter != childNodes.end(); ++iter)
 	{
 		Node *pNode = (*iter);
-		Element *pElem = dynamic_cast<Element*>(pNode);
-		if (pElem == NULL)
+		Element *pChildElem = dynamic_cast<Element*>(pNode);
+		if (pChildElem == NULL)
 		{
 			continue;
 		}
 
-		string nodeName = pElem->get_name();
-		string nodeContent = getElementContent(pElem);
+		string nodeName(pChildElem->get_name());
+		string nodeContent(getElementContent(pChildElem));
 
 		if (nodeName == "name")
 		{
@@ -858,14 +1001,14 @@ bool PinotSettings::loadColour(const Element *pElem)
 	for (Node::NodeList::iterator iter = childNodes.begin(); iter != childNodes.end(); ++iter)
 	{
 		Node *pNode = (*iter);
-		Element *pElem = dynamic_cast<Element*>(pNode);
-		if (pElem == NULL)
+		Element *pChildElem = dynamic_cast<Element*>(pNode);
+		if (pChildElem == NULL)
 		{
 			continue;
 		}
 
-		string nodeName = pElem->get_name();
-		string nodeContent = getElementContent(pElem);
+		string nodeName(pChildElem->get_name());
+		string nodeContent(getElementContent(pChildElem));
 		gushort value = (gushort)atoi(nodeContent.c_str());
 
 		if (nodeName == "red")
@@ -901,14 +1044,14 @@ bool PinotSettings::loadProxy(const Element *pElem)
 	for (Node::NodeList::iterator iter = childNodes.begin(); iter != childNodes.end(); ++iter)
 	{
 		Node *pNode = (*iter);
-		Element *pElem = dynamic_cast<Element*>(pNode);
-		if (pElem == NULL)
+		Element *pChildElem = dynamic_cast<Element*>(pNode);
+		if (pChildElem == NULL)
 		{
 			continue;
 		}
 
-		string nodeName = pElem->get_name();
-		string nodeContent = getElementContent(pElem);
+		string nodeName(pChildElem->get_name());
+		string nodeContent(getElementContent(pChildElem));
 
 		if (nodeName == "address")
 		{
@@ -957,14 +1100,14 @@ bool PinotSettings::loadIndexableLocations(const Element *pElem)
 	for (Node::NodeList::iterator iter = childNodes.begin(); iter != childNodes.end(); ++iter)
 	{
 		Node *pNode = (*iter);
-		Element *pElem = dynamic_cast<Element*>(pNode);
-		if (pElem == NULL)
+		Element *pChildElem = dynamic_cast<Element*>(pNode);
+		if (pChildElem == NULL)
 		{
 			continue;
 		}
 
-		string nodeName = pElem->get_name();
-		string nodeContent = getElementContent(pElem);
+		string nodeName(pChildElem->get_name());
+		string nodeContent(getElementContent(pChildElem));
 
 		if (nodeName == "name")
 		{
@@ -1008,14 +1151,14 @@ bool PinotSettings::loadFilePatterns(const Element *pElem)
 	for (Node::NodeList::iterator iter = childNodes.begin(); iter != childNodes.end(); ++iter)
 	{
 		Node *pNode = (*iter);
-		Element *pElem = dynamic_cast<Element*>(pNode);
-		if (pElem == NULL)
+		Element *pChildElem = dynamic_cast<Element*>(pNode);
+		if (pChildElem == NULL)
 		{
 			continue;
 		}
 
-		string nodeName = pElem->get_name();
-		string nodeContent = getElementContent(pElem);
+		string nodeName(pChildElem->get_name());
+		string nodeContent(getElementContent(pChildElem));
 
 		if (nodeName == "pattern")
 		{
@@ -1056,14 +1199,14 @@ bool PinotSettings::loadCacheProviders(const Element *pElem)
 	for (Node::NodeList::iterator iter = childNodes.begin(); iter != childNodes.end(); ++iter)
 	{
 		Node *pNode = (*iter);
-		Element *pElem = dynamic_cast<Element*>(pNode);
-		if (pElem == NULL)
+		Element *pChildElem = dynamic_cast<Element*>(pNode);
+		if (pChildElem == NULL)
 		{
 			continue;
 		}
 
-		string nodeName = pElem->get_name();
-		string nodeContent = getElementContent(pElem);
+		string nodeName(pChildElem->get_name());
+		string nodeContent(getElementContent(pChildElem));
 
 		if (nodeName == "name")
 		{
@@ -1165,164 +1308,163 @@ bool PinotSettings::save(void)
 	Element *pElem = NULL;
 	char numStr[64];
 
-	xmlpp::Document doc("1.0");
+	try
+	{
+		xmlpp::Document doc("1.0");
 
-	// Create a new node
-	pRootElem = doc.create_root_node("pinot");
-	if (pRootElem == NULL)
-	{
-		return false;
-	}
-	// ...with text children nodes
-	addChildElement(pRootElem, "version", VERSION);
-	addChildElement(pRootElem, "googleapikey", m_googleAPIKey);
-	// User interface position and size
-	pElem = pRootElem->add_child("ui");
-	if (pElem == NULL)
-	{
-		return false;
-	}
-	sprintf(numStr, "%d", m_xPos);
-	addChildElement(pElem, "xpos", numStr);
-	sprintf(numStr, "%d", m_yPos);
-	addChildElement(pElem, "ypos", numStr);
-	sprintf(numStr, "%d", m_width);
-	addChildElement(pElem, "width", numStr);
-	sprintf(numStr, "%d", m_height);
-	addChildElement(pElem, "height", numStr);
-	sprintf(numStr, "%d", m_panePos);
-	addChildElement(pElem, "panepos", numStr);
-	addChildElement(pElem, "expandqueries", (m_expandQueries ? "YES" : "NO"));
-	addChildElement(pElem, "showengines", (m_showEngines ? "YES" : "NO"));
-	// User-defined indexes
-	for (map<string, string>::iterator indexIter = m_indexNames.begin(); indexIter != m_indexNames.end(); ++indexIter)
-	{
-		string indexName = indexIter->first;
-
-		if (isInternalIndex(indexName) == true)
+		// Create a new node
+		pRootElem = doc.create_root_node("pinot");
+		if (pRootElem == NULL)
 		{
-			continue;
+			return false;
 		}
-
-		pElem = pRootElem->add_child("extraindex");
+		// ...with text children nodes
+		addChildElement(pRootElem, "version", VERSION);
+		addChildElement(pRootElem, "warnaboutversion", (m_warnAboutVersion ? "YES" : "NO"));
+		addChildElement(pRootElem, "backend", m_defaultBackend);
+		addChildElement(pRootElem, "googleapikey", m_googleAPIKey);
+		// User interface position and size
+		pElem = pRootElem->add_child("ui");
 		if (pElem == NULL)
 		{
 			return false;
 		}
-		addChildElement(pElem, "name", indexIter->first);
-		addChildElement(pElem, "location", indexIter->second);
-	}
-	// Engine channels
-	for (map<string, bool>::iterator channelIter = m_engineChannels.begin();
-		channelIter != m_engineChannels.end(); ++channelIter)
-	{
-		// Only save those whose group was collapsed
-		if (channelIter->second == false)
+		sprintf(numStr, "%d", m_xPos);
+		addChildElement(pElem, "xpos", numStr);
+		sprintf(numStr, "%d", m_yPos);
+		addChildElement(pElem, "ypos", numStr);
+		sprintf(numStr, "%d", m_width);
+		addChildElement(pElem, "width", numStr);
+		sprintf(numStr, "%d", m_height);
+		addChildElement(pElem, "height", numStr);
+		sprintf(numStr, "%d", m_panePos);
+		addChildElement(pElem, "panepos", numStr);
+		addChildElement(pElem, "expandqueries", (m_expandQueries ? "YES" : "NO"));
+		addChildElement(pElem, "showengines", (m_showEngines ? "YES" : "NO"));
+		// User-defined indexes
+		for (map<string, string>::iterator indexIter = m_indexNames.begin(); indexIter != m_indexNames.end(); ++indexIter)
 		{
-			pElem = pRootElem->add_child("channel");
+			string indexName = indexIter->first;
+
+			if (isInternalIndex(indexName) == true)
+			{
+				continue;
+			}
+
+			pElem = pRootElem->add_child("extraindex");
 			if (pElem == NULL)
 			{
 				return false;
 			}
-			addChildElement(pElem, "name", channelIter->first);
+			addChildElement(pElem, "name", indexIter->first);
+			addChildElement(pElem, "location", indexIter->second);
 		}
-	}
-	// User-defined queries
-	for (map<string, QueryProperties>::iterator queryIter = m_queries.begin();
-		queryIter != m_queries.end(); ++queryIter)
-	{
-		pElem = pRootElem->add_child("storedquery");
+		// Engine channels
+		for (map<string, bool>::iterator channelIter = m_engineChannels.begin();
+			channelIter != m_engineChannels.end(); ++channelIter)
+		{
+			// Only save those whose group was collapsed
+			if (channelIter->second == false)
+			{
+				pElem = pRootElem->add_child("channel");
+				if (pElem == NULL)
+				{
+					return false;
+				}
+				addChildElement(pElem, "name", channelIter->first);
+			}
+		}
+		// User-defined queries
+		for (map<string, QueryProperties>::iterator queryIter = m_queries.begin();
+			queryIter != m_queries.end(); ++queryIter)
+		{
+			pElem = pRootElem->add_child("storedquery");
+			if (pElem == NULL)
+			{
+				return false;
+			}
+
+			addChildElement(pElem, "name", queryIter->first);
+			addChildElement(pElem, "sortorder", (queryIter->second.getSortOrder() == QueryProperties::DATE ? "DATE" : "RELEVANCE"));
+			addChildElement(pElem, "text", queryIter->second.getFreeQuery());
+			addChildElement(pElem, "stemlanguage", Languages::toEnglish(queryIter->second.getStemmingLanguage()));
+			sprintf(numStr, "%u", queryIter->second.getMaximumResultsCount());
+			addChildElement(pElem, "maxresults", numStr);
+			addChildElement(pElem, "index", (queryIter->second.getIndexResults() ? "ALL" : "NONE"));
+			addChildElement(pElem, "label", queryIter->second.getLabelName());
+			addChildElement(pElem, "modified", (queryIter->second.getModified() ? "YES" : "NO"));
+		}
+		// Labels
+		for (set<string>::iterator labelIter = m_labels.begin(); labelIter != m_labels.end(); ++labelIter)
+		{
+			pElem = pRootElem->add_child("label");
+			if (pElem == NULL)
+			{
+				return false;
+			}
+			addChildElement(pElem, "name", *labelIter);
+		}
+		// Ignore robots directives
+		addChildElement(pRootElem, "robots", (m_ignoreRobotsDirectives ? "IGNORE" : "OBEY"));
+		// Enable terms suggestion
+		addChildElement(pRootElem, "suggestterms", (m_suggestQueryTerms ? "YES" : "NO"));
+		// New results colour
+		pElem = pRootElem->add_child("newresults");
 		if (pElem == NULL)
 		{
 			return false;
 		}
-
-		Date aDate;
-		unsigned int day, month, year;
-
-		addChildElement(pElem, "name", queryIter->first);
-		addChildElement(pElem, "text", queryIter->second.getFreeQuery());
-		sprintf(numStr, "%u", queryIter->second.getMaximumResultsCount());
-		addChildElement(pElem, "maxresults", numStr);
-		bool enable = queryIter->second.getMinimumDate(day, month, year);
-		if (year > 0)
-		{
-			addChildElement(pElem, "enablemindate", (enable ? "YES" : "NO"));
-			aDate.set_dmy((Date::Day )day, (Date::Month )month, (Date::Year )year);
-			addChildElement(pElem, "mindate", aDate.format_string("%F"));
-		}
-		enable = queryIter->second.getMaximumDate(day, month, year);
-		if (year > 0)
-		{
-			addChildElement(pElem, "enablemaxdate", (enable ? "YES" : "NO"));
-			aDate.set_dmy((Date::Day )day, (Date::Month )month, (Date::Year )year);
-			addChildElement(pElem, "maxdate", aDate.format_string("%F"));
-		}
-		addChildElement(pElem, "index", (queryIter->second.getIndexResults() ? "ALL" : "NONE"));
-		addChildElement(pElem, "label", queryIter->second.getLabelName());
-	}
-	// Labels
-	for (set<string>::iterator labelIter = m_labels.begin(); labelIter != m_labels.end(); ++labelIter)
-	{
-		pElem = pRootElem->add_child("label");
+		sprintf(numStr, "%u", m_newResultsColourRed);
+		addChildElement(pElem, "red", numStr);
+		sprintf(numStr, "%u", m_newResultsColourGreen);
+		addChildElement(pElem, "green", numStr);
+		sprintf(numStr, "%u", m_newResultsColourBlue);
+		addChildElement(pElem, "blue", numStr);
+		// Proxy
+		pElem = pRootElem->add_child("proxy");
 		if (pElem == NULL)
 		{
 			return false;
 		}
-		addChildElement(pElem, "name", *labelIter);
-	}
-	// Ignore robots directives
-	addChildElement(pRootElem, "robots", (m_ignoreRobotsDirectives ? "IGNORE" : "OBEY"));
-	// Enable terms suggestion
-	addChildElement(pRootElem, "suggestterms", (m_suggestQueryTerms ? "YES" : "NO"));
-	// New results colour
-	pElem = pRootElem->add_child("newresults");
-	if (pElem == NULL)
-	{
-		return false;
-	}
-	sprintf(numStr, "%u", m_newResultsColourRed);
-	addChildElement(pElem, "red", numStr);
-	sprintf(numStr, "%u", m_newResultsColourGreen);
-	addChildElement(pElem, "green", numStr);
-	sprintf(numStr, "%u", m_newResultsColourBlue);
-	addChildElement(pElem, "blue", numStr);
-	// Proxy
-	pElem = pRootElem->add_child("proxy");
-	if (pElem == NULL)
-	{
-		return false;
-	}
-	addChildElement(pElem, "address", m_proxyAddress);
-	sprintf(numStr, "%u", m_proxyPort);
-	addChildElement(pElem, "port", numStr);
-	addChildElement(pElem, "type", m_proxyType);
-	addChildElement(pElem, "enable", (m_proxyEnabled ? "YES" : "NO"));
-	// Locations to index 
-	for (set<IndexableLocation>::iterator locationIter = m_indexableLocations.begin(); locationIter != m_indexableLocations.end(); ++locationIter)
-	{
-		pElem = pRootElem->add_child("indexable");
+		addChildElement(pElem, "address", m_proxyAddress);
+		sprintf(numStr, "%u", m_proxyPort);
+		addChildElement(pElem, "port", numStr);
+		addChildElement(pElem, "type", m_proxyType);
+		addChildElement(pElem, "enable", (m_proxyEnabled ? "YES" : "NO"));
+		// Locations to index 
+		for (set<IndexableLocation>::iterator locationIter = m_indexableLocations.begin();
+			locationIter != m_indexableLocations.end(); ++locationIter)
+		{
+			pElem = pRootElem->add_child("indexable");
+			if (pElem == NULL)
+			{
+				return false;
+			}
+			addChildElement(pElem, "name", locationIter->m_name);
+			addChildElement(pElem, "monitor", (locationIter->m_monitor ? "YES" : "NO"));
+		}
+		// File patterns
+		pElem = pRootElem->add_child("patterns");
 		if (pElem == NULL)
 		{
 			return false;
 		}
-		addChildElement(pElem, "name", locationIter->m_name);
-		addChildElement(pElem, "monitor", (locationIter->m_monitor ? "YES" : "NO"));
+		for (set<ustring>::iterator patternIter = m_filePatternsList.begin();
+			patternIter != m_filePatternsList.end() ; ++patternIter)
+		{
+			addChildElement(pElem, "pattern", *patternIter);
+		}
+		addChildElement(pElem, "forbid", (m_isBlackList ? "YES" : "NO"));
+
+		// Save to file
+		doc.write_to_file_formatted(getConfigurationFileName());
 	}
-	// File patterns
-	pElem = pRootElem->add_child("patterns");
-	if (pElem == NULL)
+	catch (const std::exception& ex)
 	{
+		cerr << "Couldn't save configuration file: "
+			<< ex.what() << endl;
 		return false;
 	}
-	for (set<ustring>::iterator patternIter = m_filePatternsList.begin(); patternIter != m_filePatternsList.end() ; ++patternIter)
-	{
-		addChildElement(pElem, "pattern", *patternIter);
-	}
-	addChildElement(pElem, "forbid", (m_isBlackList ? "YES" : "NO"));
-
-	// Save to file
-	doc.write_to_file_formatted(getConfigurationFileName());
 
 	return true;
 }
@@ -1397,9 +1539,10 @@ void PinotSettings::clearIndexes(void)
 }
 
 /// Returns an ID that identifies the given index.
-unsigned int PinotSettings::getIndexId(const std::string &name)
+unsigned int PinotSettings::getIndexIdByName(const std::string &name)
 {
 	unsigned int indexId = 0;
+
 	for (map<unsigned int, string>::iterator mapIter = m_indexIds.begin();
 		mapIter != m_indexIds.end(); ++mapIter)
 	{
@@ -1413,6 +1556,21 @@ unsigned int PinotSettings::getIndexId(const std::string &name)
 	return indexId;
 }
 
+/// Returns an ID that identifies the given index.
+unsigned int PinotSettings::getIndexIdByLocation(const std::string &location)
+{
+	for (map<string, string>::iterator mapIter = m_indexNames.begin();
+		mapIter != m_indexNames.end(); ++mapIter)
+	{
+		if (mapIter->second == location)
+		{
+			return getIndexIdByName(mapIter->first);
+		}
+	}
+
+	return 0;
+}
+
 /// Returns the name(s) for the given ID.
 void PinotSettings::getIndexNames(unsigned int id, std::set<std::string> &names)
 {
@@ -1446,15 +1604,15 @@ IndexInterface *PinotSettings::getIndex(const string &location)
 {
 	if (location == m_docsIndexLocation)
 	{
-		return IndexFactory::getIndex("xapian", m_docsIndexLocation);
+		return ModuleFactory::getIndex(m_defaultBackend, m_docsIndexLocation);
 	}
 	else if ((m_clientMode == true) &&
 		(location == m_daemonIndexLocation))
 	{
-		return IndexFactory::getIndex("dbus", m_daemonIndexLocation);
+		return ModuleFactory::getIndex("dbus-" + m_defaultBackend, m_daemonIndexLocation);
 	}
 
-	return IndexFactory::getIndex("xapian", location);
+	return ModuleFactory::getIndex(m_defaultBackend, location);
 }
 
 /// Returns the search engines set.
@@ -1546,7 +1704,7 @@ const map<string, QueryProperties> &PinotSettings::getQueries(void) const
 /// Adds a new query.
 bool PinotSettings::addQuery(const QueryProperties &properties)
 {
-	string name = properties.getName();
+	string name(properties.getName());
 
 	map<string, QueryProperties>::iterator queryIter = m_queries.find(name);
 	if (queryIter == m_queries.end())
@@ -1581,34 +1739,6 @@ void PinotSettings::clearQueries(void)
 	m_queries.clear();
 }
 
-/// Returns the labels list.
-const set<string> &PinotSettings::getLabels(void) const
-{
-	return m_labels;
-}
-
-/// Adds a new label.
-void PinotSettings::addLabel(const string &name)
-{
-	m_labels.insert(name);
-}
-
-/// Removes a label.
-void PinotSettings::removeLabel(const string &name)
-{
-	set<string>::iterator labelIter = m_labels.find(name);
-	if (labelIter != m_labels.end())
-	{
-		m_labels.erase(labelIter);
-	}
-}
-
-/// Clears the labels list.
-void PinotSettings::clearLabels(void)
-{
-	m_labels.clear();
-}
-
 /// Determines if a file matches the blacklist.
 bool PinotSettings::isBlackListed(const string &fileName)
 {
@@ -1744,13 +1874,15 @@ bool PinotSettings::TimestampedItem::operator==(const TimestampedItem &other) co
 }
 
 PinotSettings::IndexableLocation::IndexableLocation() :
-	m_monitor(false)
+	m_monitor(false),
+	m_isSource(true)
 {
 }
 
 PinotSettings::IndexableLocation::IndexableLocation(const IndexableLocation &other) :
 	m_monitor(other.m_monitor),
-	m_name(other.m_name)
+	m_name(other.m_name),
+	m_isSource(other.m_isSource)
 {
 }
 
@@ -1764,6 +1896,7 @@ PinotSettings::IndexableLocation &PinotSettings::IndexableLocation::operator=(co
 	{
 		m_monitor = other.m_monitor;
 		m_name = other.m_name;
+		m_isSource = other.m_isSource;
 	}
 
 	return *this;
