@@ -40,6 +40,8 @@ class PinotSettings
 
 		static bool enableClientMode(bool enable);
 
+		static std::string getHomeDirectory(void);
+
 		static std::string getConfigurationDirectory(void);
 
 		static std::string getConfigurationFileName(void);
@@ -78,7 +80,10 @@ class PinotSettings
 		void clearIndexes(void);
 
 		/// Returns an ID that identifies the given index.
-		unsigned int getIndexId(const std::string &name);
+		unsigned int getIndexIdByName(const std::string &name);
+
+		/// Returns an ID that identifies the given index.
+		unsigned int getIndexIdByLocation(const std::string &location);
 
 		/// Returns the name(s) for the given ID.
 		void getIndexNames(unsigned int id, std::set<std::string> &names);
@@ -128,18 +133,6 @@ class PinotSettings
 		/// Clears the queries map.
 		void clearQueries(void);
 
-		/// Returns the labels list.
-		const std::set<std::string> &getLabels(void) const;
-
-		/// Adds a new label.
-		void addLabel(const std::string &name);
-
-		/// Removes a label.
-		void removeLabel(const std::string &name);
-
-		/// Clears the labels list.
-		void clearLabels(void);
-
 		/// Determines if a file matches the blacklist.
 		bool isBlackListed(const std::string &fileName);
 
@@ -171,6 +164,7 @@ class PinotSettings
 
 				bool m_monitor;
 				Glib::ustring m_name;
+				bool m_isSource;
 
 		};
 
@@ -191,9 +185,12 @@ class PinotSettings
 		};
 
 		Glib::ustring m_version;
+		bool m_warnAboutVersion;
+		Glib::ustring m_defaultBackend;
 		Glib::ustring m_googleAPIKey;
 		Glib::ustring m_docsIndexLocation;
 		Glib::ustring m_daemonIndexLocation;
+		double m_minimumDiskSpace;
 		int m_xPos;
 		int m_yPos;
 		int m_width;
@@ -210,6 +207,7 @@ class PinotSettings
 		unsigned int m_proxyPort;
 		Glib::ustring  m_proxyType;
 		bool m_proxyEnabled;
+		std::set<std::string> m_labels;
 		std::set<IndexableLocation> m_indexableLocations;
 		std::set<Glib::ustring> m_filePatternsList;
 		bool m_isBlackList;
@@ -226,7 +224,6 @@ class PinotSettings
 		std::map<unsigned int, std::string> m_engineIds;
 		std::map<std::string, bool> m_engineChannels;
 		std::map<std::string, QueryProperties> m_queries;
-		std::set<std::string> m_labels;
 
 		PinotSettings();
 		bool loadConfiguration(const std::string &fileName, bool isGlobal);
