@@ -40,7 +40,6 @@
 #include "ResultsTree.h"
 
 using namespace std;
-using namespace SigC;
 using namespace Glib;
 using namespace Gdk;
 using namespace Gtk;
@@ -106,15 +105,15 @@ ResultsTree::ResultsTree(const ustring &queryName, Menu *pPopupMenu,
 		// Pack an icon renderer for the viewed status
 		CellRendererPixbuf *pIconRenderer = new CellRendererPixbuf();
 		pColumn->pack_start(*manage(pIconRenderer), false);
-		pColumn->set_cell_data_func(*pIconRenderer, SigC::slot(*this, &ResultsTree::renderViewStatus));
+		pColumn->set_cell_data_func(*pIconRenderer, sigc::mem_fun(*this, &ResultsTree::renderViewStatus));
 		// Pack a second icon renderer for the indexed status
 		pIconRenderer = new CellRendererPixbuf();
 		pColumn->pack_start(*manage(pIconRenderer), false);
-		pColumn->set_cell_data_func(*pIconRenderer, SigC::slot(*this, &ResultsTree::renderIndexStatus));
+		pColumn->set_cell_data_func(*pIconRenderer, sigc::mem_fun(*this, &ResultsTree::renderIndexStatus));
 		// And a third one for the ranking
 		pIconRenderer = new CellRendererPixbuf();
 		pColumn->pack_start(*manage(pIconRenderer), false);
-		pColumn->set_cell_data_func(*pIconRenderer, SigC::slot(*this, &ResultsTree::renderRanking));
+		pColumn->set_cell_data_func(*pIconRenderer, sigc::mem_fun(*this, &ResultsTree::renderRanking));
 		pColumn->set_sizing(TREE_VIEW_COLUMN_AUTOSIZE);
 		append_column(*manage(pColumn));
 
@@ -133,7 +132,7 @@ ResultsTree::ResultsTree(const ustring &queryName, Menu *pPopupMenu,
 	pColumn = new TreeViewColumn(_("Title"));
 	CellRendererText *pTextRenderer = new CellRendererText();
 	pColumn->pack_start(*manage(pTextRenderer));
-	pColumn->set_cell_data_func(*pTextRenderer, SigC::slot(*this, &ResultsTree::renderTitleColumn));
+	pColumn->set_cell_data_func(*pTextRenderer, sigc::mem_fun(*this, &ResultsTree::renderTitleColumn));
 	pColumn->add_attribute(pTextRenderer->property_text(), m_resultsColumns.m_text);
 	pColumn->set_resizable(true);
 	pColumn->set_sort_column(m_resultsColumns.m_text);
@@ -157,16 +156,16 @@ ResultsTree::ResultsTree(const ustring &queryName, Menu *pPopupMenu,
 	
 	// Connect the signals
 	signal_button_press_event().connect_notify(
-		SigC::slot(*this, &ResultsTree::onButtonPressEvent));
+		sigc::mem_fun(*this, &ResultsTree::onButtonPressEvent));
 	get_selection()->signal_changed().connect(
-		SigC::slot(*this, &ResultsTree::onSelectionChanged));
+		sigc::mem_fun(*this, &ResultsTree::onSelectionChanged));
 
 	// Enable interactive search
 	set_search_column(m_resultsColumns.m_text.index());
 	// Control which rows can be selected
-	get_selection()->set_select_function(SigC::slot(*this, &ResultsTree::onSelectionSelect));
+	get_selection()->set_select_function(sigc::mem_fun(*this, &ResultsTree::onSelectionSelect));
 	// Listen for style changes
-	signal_style_changed().connect_notify(SigC::slot(*this, &ResultsTree::onStyleChanged));
+	signal_style_changed().connect_notify(sigc::mem_fun(*this, &ResultsTree::onStyleChanged));
 
 	// Render the icons
 	m_indexedIconPixbuf = render_icon(Stock::INDEX, ICON_SIZE_MENU, "MetaSE-pinot");
@@ -180,7 +179,7 @@ ResultsTree::ResultsTree(const ustring &queryName, Menu *pPopupMenu,
 	pColumn = new TreeViewColumn(_("Extract"));
 	pTextRenderer = new CellRendererText();
 	pColumn->pack_start(*manage(pTextRenderer));
-	pColumn->set_cell_data_func(*pTextRenderer, SigC::slot(*this, &ResultsTree::renderExtractColumn));
+	pColumn->set_cell_data_func(*pTextRenderer, sigc::mem_fun(*this, &ResultsTree::renderExtractColumn));
 	pColumn->add_attribute(pTextRenderer->property_text(), m_extractColumns.m_name);
 	pColumn->set_sizing(TREE_VIEW_COLUMN_AUTOSIZE);
 	m_extractTreeView->append_column(*manage(pColumn));
@@ -368,9 +367,9 @@ void ResultsTree::onSelectionChanged(void)
 }
 
 bool ResultsTree::onSelectionSelect(const RefPtr<TreeModel>& model,
-	const TreeModel::Path& path, bool path_currently_selected)
+	const TreeModel::Path& node_path, bool path_currently_selected)
 {
-	const TreeModel::iterator iter = model->get_iter(path);
+	const TreeModel::iterator iter = model->get_iter(node_path);
 	const TreeModel::Row row = *iter;
 
 	m_indexNames.clear();
@@ -389,8 +388,8 @@ bool ResultsTree::onSelectionSelect(const RefPtr<TreeModel>& model,
 			cout << "ResultsTree::onSelectionSelect: extract for " << row[m_resultsColumns.m_text] << endl;
 #endif
 
-			TreeModel::iterator iter = m_refExtractStore->append();
-			TreeModel::Row extractRow = *iter;
+			TreeModel::iterator extractIter = m_refExtractStore->append();
+			TreeModel::Row extractRow = *extractIter;
 			extractRow[m_extractColumns.m_name] = findResultsExtract(row);
 		}
 	}
@@ -458,7 +457,7 @@ ustring ResultsTree::getExtract(void) const
 // Returns true if something was added to the tree.
 //
 bool ResultsTree::addResults(const string &engineName, const vector<DocumentInfo> &resultsList,
-	const string &charset)
+	const string &charset, bool updateHistory)
 {
 	std::map<string, TreeModel::iterator> updatedGroups;
 	ResultsModelColumns::RowType rootType;
@@ -486,7 +485,7 @@ bool ResultsTree::addResults(const string &engineName, const vector<DocumentInfo
 		if (mapIter != m_settings.getIndexes().end())
 		{
 			// Yes, it is
-			indexId = m_settings.getIndexId(engineName);
+			indexId = m_settings.getIndexIdByName(engineName);
 			engineId = m_settings.getEngineId("Xapian");
 #ifdef DEBUG
 			cout << "ResultsTree::addResults: engine is index " << engineName << " " << indexId << endl;
@@ -500,16 +499,25 @@ bool ResultsTree::addResults(const string &engineName, const vector<DocumentInfo
 	cout << "ResultsTree::addResults: ID for engine " << engineName << " is " << engineId <<  endl;
 #endif
 
-	QueryHistory history(m_settings.getHistoryDatabaseName());
+	QueryHistory queryHistory(m_settings.getHistoryDatabaseName());
+	ViewHistory viewHistory(m_settings.getHistoryDatabaseName());
+	string lastRun(queryHistory.getLastRun(m_treeName, engineName));
+	time_t lastRunTime = 0;
 	bool isNewQuery = false;
-	if (history.getLastRun(m_treeName, engineName).empty() == true)
+
+	// Is this a new query ?
+	if (lastRun.empty() == true)
 	{
 		isNewQuery = true;
 	}
+	else
+	{
+		lastRunTime = TimeConverter::fromTimestamp(lastRun);
+	}
 
 	// Look at the results list
 #ifdef DEBUG
-	cout << "ResultsTree::addResults: " << resultsList.size() << " results" << endl;
+	cout << "ResultsTree::addResults: " << resultsList.size() << " results, last run " << lastRun << endl;
 #endif
 	for (vector<DocumentInfo>::const_iterator resultIter = resultsList.begin();
 		resultIter != resultsList.end(); ++resultIter)
@@ -539,20 +547,26 @@ bool ResultsTree::addResults(const string &engineName, const vector<DocumentInfo
 
 			// Has the result's ranking changed ?
 			float oldestScore = 0;
-			float previousScore = history.hasItem(m_treeName, engineName,
+			float previousScore = queryHistory.hasItem(m_treeName, engineName,
 				location, oldestScore);
 			if (previousScore > 0)
 			{
-				// Update this result whatever the current and previous rankings were
-				history.updateItem(m_treeName, engineName, location,
-					title, extract, charset, currentScore);
+				if (updateHistory == true)
+				{
+					// Update this result whatever the current and previous rankings were
+					queryHistory.updateItem(m_treeName, engineName, location,
+						title, extract, charset, currentScore);
+				}
 				rankDiff = (int)(currentScore - previousScore);
 			}
 			else
 			{
 				// No, this is a new result
-				history.insertItem(m_treeName, engineName, location,
-					resultIter->getTitle(), extract, charset, currentScore);
+				if (updateHistory == true)
+				{
+					queryHistory.insertItem(m_treeName, engineName, location,
+						resultIter->getTitle(), extract, charset, currentScore);
+				}
 				// New results are displayed as such only if the query has already been run on the engine
 				if (isNewQuery == false)
 				{
@@ -572,10 +586,14 @@ bool ResultsTree::addResults(const string &engineName, const vector<DocumentInfo
 			isIndexed = true;
 		}
 
+		// Has it been already viewed ?
+		bool wasViewed = viewHistory.hasItem(location);
+
 		// OK, add a row for this result within the group
 		TreeModel::iterator titleIter;
-		if (appendResult(title, location, resultIter->getType(), (int)currentScore, rankDiff, isIndexed,
-			docId, resultIter->getTimestamp(), engineId, indexId, titleIter, groupIter, true) == true)
+		if (appendResult(title, location, (int)currentScore, rankDiff, isIndexed, wasViewed,
+			docId, resultIter->getTimestamp(), resultIter->serialize(),
+			engineId, indexId, titleIter, groupIter, true) == true)
 		{
 #ifdef DEBUG
 			cout << "ResultsTree::addResults: added row for result " << count
@@ -591,6 +609,17 @@ bool ResultsTree::addResults(const string &engineName, const vector<DocumentInfo
 		}
 	}
 
+	// Remove older items ?
+	if ((isNewQuery == false) &&
+		(updateHistory == true))
+	{
+#ifdef DEBUG
+		cout << "ResultsTree::addResults: removing items for " << m_treeName
+			<< ", " << engineName << " older than " << lastRunTime << endl;
+#endif
+		queryHistory.deleteItems(m_treeName, engineName, lastRunTime);
+	}
+
 	if (count > 0)
 	{
 #ifdef DEBUG
@@ -711,12 +740,13 @@ void ResultsTree::setGroupMode(GroupByMode groupMode)
 					// Add result
 					success = appendResult(childRow[m_resultsColumns.m_text],
 						childRow[m_resultsColumns.m_url],
-						childRow[m_resultsColumns.m_type],
 						childRow[m_resultsColumns.m_score],
 						childRow[m_resultsColumns.m_rankDiff],
 						childRow[m_resultsColumns.m_indexed],
+						childRow[m_resultsColumns.m_viewed],
 						childRow[m_resultsColumns.m_docId],
 						childRow[m_resultsColumns.m_timestamp],
+						childRow[m_resultsColumns.m_serial],
 						engineIds, indexIds,
 						newIter, groupIter, true);
 				}
@@ -745,10 +775,10 @@ void ResultsTree::setGroupMode(GroupByMode groupMode)
 						// Add entries for each index name so that we can loop once on engine names
 						set<string> indexNames;
 						m_settings.getIndexNames(indexIds, indexNames);
-						for (set<string>::iterator iter = indexNames.begin();
-							iter != indexNames.end(); ++iter)
+						for (set<string>::iterator indexIter = indexNames.begin();
+							indexIter != indexNames.end(); ++indexIter)
 						{
-							string indexName(*iter);
+							string indexName(*indexIter);
 							engineNames.insert(indexName);
 #ifdef DEBUG
 							cout << "ResultsTree::setGroupMode: row is for index " << indexName << endl;
@@ -756,17 +786,17 @@ void ResultsTree::setGroupMode(GroupByMode groupMode)
 						}
 					}
 
-					for (set<string>::iterator iter = engineNames.begin();
-						iter != engineNames.end(); ++iter)
+					for (set<string>::iterator engineIter = engineNames.begin();
+						engineIter != engineNames.end(); ++engineIter)
 					{
-						string engineName(*iter);
+						string engineName(*engineIter);
 						unsigned int indexId = 0;
 						unsigned int engineId = m_settings.getEngineId(engineName);
 
 						if (engineId == 0)
 						{
 							// This is actually an index, not an engine...
-							indexId = m_settings.getIndexId(engineName);
+							indexId = m_settings.getIndexIdByName(engineName);
 							if (indexId > 0)
 							{
 								engineId = m_settings.getEngineId("Xapian");
@@ -785,16 +815,17 @@ void ResultsTree::setGroupMode(GroupByMode groupMode)
 							// Add result
 							appendResult(childRow[m_resultsColumns.m_text],
 								childRow[m_resultsColumns.m_url],
-								childRow[m_resultsColumns.m_type],
 								childRow[m_resultsColumns.m_score],
 								childRow[m_resultsColumns.m_rankDiff],
 								childRow[m_resultsColumns.m_indexed],
+								childRow[m_resultsColumns.m_viewed],
 								childRow[m_resultsColumns.m_docId],
 								childRow[m_resultsColumns.m_timestamp],
+								childRow[m_resultsColumns.m_serial],
 								engineId, indexId,
 								newIter, groupIter, true);
 #ifdef DEBUG
-							cout << "ResultsTree::setGroupMode: row for " << *iter << endl;
+							cout << "ResultsTree::setGroupMode: row for " << engineName << endl;
 #endif
 						}
 					}
@@ -880,9 +911,10 @@ bool ResultsTree::getSelection(vector<DocumentInfo> &resultsList, bool skipIndex
 		if ((skipIndexed == false) ||
 			(isIndexed == false))
 		{
-			DocumentInfo current(from_utf8(row[m_resultsColumns.m_text]),
-				from_utf8(row[m_resultsColumns.m_url]),
-				from_utf8(row[m_resultsColumns.m_type]), "");
+			DocumentInfo current;
+			string serial(from_utf8(row[m_resultsColumns.m_serial]));
+
+			current.deserialize(serial);
 
 			if (isIndexed == true)
 			{
@@ -898,7 +930,7 @@ bool ResultsTree::getSelection(vector<DocumentInfo> &resultsList, bool skipIndex
 #ifdef DEBUG
 						cout << "ResultsTree::getSelection: result in internal index " << *indexIter << endl;
 #endif
-						current.setIsIndexed(m_settings.getIndexId(*indexIter), row[m_resultsColumns.m_docId]);
+						current.setIsIndexed(m_settings.getIndexIdByName(*indexIter), row[m_resultsColumns.m_docId]);
 						break;
 					}
 				}
@@ -948,14 +980,14 @@ void ResultsTree::setSelectionState(bool viewed)
 //
 // Updates a result's properties.
 //
-void ResultsTree::updateResult(const DocumentInfo &result)
+bool ResultsTree::updateResult(const DocumentInfo &result)
 {
 	unsigned int indexId = 0;
 	unsigned int docId = result.getIsIndexed(indexId);
 
 	if (docId == 0)
 	{
-		return;
+		return false;
 	}
 
 	// Go through the list
@@ -966,15 +998,19 @@ void ResultsTree::updateResult(const DocumentInfo &result)
 
 		if (docId == row[m_resultsColumns.m_docId])
 		{
-			updateRow(row, result.getTitle(), result.getLocation(), result.getType(),
+			updateRow(row, result.getTitle(), result.getLocation(),
 				row[m_resultsColumns.m_score], row[m_resultsColumns.m_engines],
 				row[m_resultsColumns.m_indexes], docId,
-				result.getTimestamp(), ResultsModelColumns::ROW_RESULT,
+				result.getTimestamp(), result.serialize(),
+				ResultsModelColumns::ROW_RESULT,
 				row[m_resultsColumns.m_indexed], row[m_resultsColumns.m_viewed],
 				row[m_resultsColumns.m_rankDiff]);
-			break;
+
+			return true;
 		}
 	}
+
+	return false;
 }
 
 //
@@ -1054,7 +1090,7 @@ bool ResultsTree::deleteResults(const string &engineName)
 		if (mapIter != m_settings.getIndexes().end())
 		{
 			// Yes, it is
-			indexId = m_settings.getIndexId(engineName);
+			indexId = m_settings.getIndexIdByName(engineName);
 			engineId = m_settings.getEngineId("Xapian");
 		}
 	}
@@ -1165,9 +1201,9 @@ void ResultsTree::clear(void)
 //
 // Shows or hides the extract field.
 //
-void ResultsTree::showExtract(bool show)
+void ResultsTree::showExtract(bool showExtract)
 {
-	m_showExtract = show;
+	m_showExtract = showExtract;
 	if (m_showExtract == true)
 	{
 		// Show the extract
@@ -1185,7 +1221,7 @@ void ResultsTree::showExtract(bool show)
 //
 void ResultsTree::exportResults(const string &fileName, bool csvFormat)
 {
-	QueryHistory history(m_settings.getHistoryDatabaseName());
+	QueryHistory queryHistory(m_settings.getHistoryDatabaseName());
 	QueryProperties queryProps(m_treeName, "");
 	ResultsExporter *pExporter = NULL;
 	unsigned int maxResultsCount = 0;
@@ -1244,17 +1280,16 @@ void ResultsTree::exportResults(const string &fileName, bool csvFormat)
 			childIter != groupChildren.end(); ++childIter)
 		{
 			set<string> engineNames, indexNames;
-			TreeModel::Row row = *childIter;
-			DocumentInfo result(from_utf8(row[m_resultsColumns.m_text]),
-				from_utf8(row[m_resultsColumns.m_url]),
-				from_utf8(row[m_resultsColumns.m_type]), "");
-			string engineName, charset;
-			unsigned int engineIds = row[m_resultsColumns.m_engines];
-			unsigned int indexIds = row[m_resultsColumns.m_indexes];
+			TreeModel::Row childRow = *childIter;
+			DocumentInfo result;
+			string engineName, charset, serial(from_utf8(childRow[m_resultsColumns.m_serial]));
+			unsigned int engineIds = childRow[m_resultsColumns.m_engines];
+			unsigned int indexIds = childRow[m_resultsColumns.m_indexes];
 
 #ifdef DEBUG
 			cout << "ResultsTree::exportResults: engines " << engineIds << ", indexes " << indexIds << endl;
 #endif
+			result.deserialize(serial);
 			m_settings.getEngineNames(engineIds, engineNames);
 			if (engineNames.empty() == false)
 			{
@@ -1272,9 +1307,10 @@ void ResultsTree::exportResults(const string &fileName, bool csvFormat)
 			}
 			if (m_groupMode != FLAT)
 			{
-				result.setExtract(history.getItemExtract(from_utf8(m_treeName),
+				result.setExtract(queryHistory.getItemExtract(from_utf8(m_treeName),
 					engineName, result.getLocation(), charset));
 			}
+			result.setTimestamp(from_utf8(childRow[m_resultsColumns.m_timestamp]));
 
 			// Export this
 			pExporter->exportResult(engineName, result);
@@ -1290,7 +1326,7 @@ void ResultsTree::exportResults(const string &fileName, bool csvFormat)
 //
 // Returns the changed selection signal.
 //
-Signal1<void, ustring>& ResultsTree::getSelectionChangedSignal(void)
+sigc::signal1<void, ustring>& ResultsTree::getSelectionChangedSignal(void)
 {
 	return m_signalSelectionChanged;
 }
@@ -1298,7 +1334,7 @@ Signal1<void, ustring>& ResultsTree::getSelectionChangedSignal(void)
 //
 // Returns the double-click signal.
 //
-Signal0<void>& ResultsTree::getDoubleClickSignal(void)
+sigc::signal0<void>& ResultsTree::getDoubleClickSignal(void)
 {
 	return m_signalDoubleClick;
 }
@@ -1307,8 +1343,9 @@ Signal0<void>& ResultsTree::getDoubleClickSignal(void)
 // Adds a new row in the results tree.
 //
 bool ResultsTree::appendResult(const ustring &text, const ustring &url,
-	const ustring &type, int score, int rankDiff, bool isIndexed,
-	unsigned int docId, const ustring &timestamp, unsigned int engineId,
+	int score, int rankDiff, bool isIndexed, bool wasViewed,
+	unsigned int docId, const ustring &timestamp,
+	const ustring &serial, unsigned int engineId,
 	unsigned int indexId, TreeModel::iterator &newRowIter,
 	const TreeModel::iterator &parentIter, bool noDuplicates)
 {
@@ -1353,13 +1390,9 @@ bool ResultsTree::appendResult(const ustring &text, const ustring &url,
 		newRowIter = m_refStore->append(parentRow.children());
 	}
 
-	// Has it been already viewed ?
-	ViewHistory viewHistory(m_settings.getHistoryDatabaseName());
-	bool wasViewed = viewHistory.hasItem(url);
-
 	TreeModel::Row childRow = *newRowIter;
-	updateRow(childRow, text, url, type, score, engineId, indexId,
-		docId, timestamp, ResultsModelColumns::ROW_RESULT, isIndexed,
+	updateRow(childRow, text, url, score, engineId, indexId,
+		docId, timestamp, serial, ResultsModelColumns::ROW_RESULT, isIndexed,
 		wasViewed, rankDiff);
 
 	return true;
@@ -1381,7 +1414,7 @@ bool ResultsTree::appendGroup(const string &groupName, ResultsModelColumns::RowT
 		groupIter = m_refStore->append();
 		TreeModel::Row groupRow = *groupIter;
 		updateRow(groupRow, to_utf8(groupName),
-			"", "", 0, 0, 0, 0, "", groupType,
+			"", 0, 0, 0, 0, "", "", groupType,
 			false, false, false);
 
 		// Update the map
@@ -1449,16 +1482,14 @@ void ResultsTree::updateGroup(TreeModel::iterator &groupIter)
 // Updates a row.
 //
 void ResultsTree::updateRow(TreeModel::Row &row, const ustring &text,
-	const ustring &url, const ustring &type, int score,
-	unsigned int engineId, unsigned int indexId,
-	unsigned int docId, const ustring &timestamp,
+	const ustring &url, int score, 	unsigned int engineId, unsigned int indexId,
+	unsigned int docId, const ustring &timestamp, const ustring &serial,
 	ResultsModelColumns::RowType resultType, bool indexed, bool viewed, int rankDiff)
 {
 	try
 	{
 		row[m_resultsColumns.m_text] = text;
 		row[m_resultsColumns.m_url] = url;
-		row[m_resultsColumns.m_type] = type;
 		row[m_resultsColumns.m_score] = score;
 		row[m_resultsColumns.m_scoreText] = "";
 		row[m_resultsColumns.m_engines] = engineId;
@@ -1467,6 +1498,7 @@ void ResultsTree::updateRow(TreeModel::Row &row, const ustring &text,
 		row[m_resultsColumns.m_resultType] = resultType;
 		row[m_resultsColumns.m_timestamp] = timestamp;
 		row[m_resultsColumns.m_timestampTime] = TimeConverter::fromTimestamp(timestamp);
+		row[m_resultsColumns.m_serial] = serial;
 
 		row[m_resultsColumns.m_indexed] = indexed;
 		row[m_resultsColumns.m_viewed] = viewed;
@@ -1491,7 +1523,7 @@ void ResultsTree::updateRow(TreeModel::Row &row, const ustring &text,
 //
 ustring ResultsTree::findResultsExtract(const Gtk::TreeModel::Row &row)
 {
-	QueryHistory history(m_settings.getHistoryDatabaseName());
+	QueryHistory queryHistory(m_settings.getHistoryDatabaseName());
 	set<string> engineNames, indexNames;
 	string url(from_utf8(row[m_resultsColumns.m_url]));
 	string extract, charset;
@@ -1528,7 +1560,7 @@ ustring ResultsTree::findResultsExtract(const Gtk::TreeModel::Row &row)
 #ifdef DEBUG
 		cout << "ResultsTree::findResultsExtract: first engine for " << url << " was " << engineName << endl;
 #endif
-		extract = history.getItemExtract(from_utf8(m_treeName), engineName, url, charset);
+		extract = queryHistory.getItemExtract(from_utf8(m_treeName), engineName, url, charset);
 	}
 
 	return to_utf8(extract, charset);
