@@ -23,7 +23,7 @@
 #include <vector>
 #include <set>
 #include <map>
-#include <sigc++/slot.h>
+#include <sigc++/sigc++.h>
 #include <glibmm/refptr.h>
 #include <glibmm/ustring.h>
 #include <gdkmm/pixbuf.h>
@@ -64,7 +64,7 @@ class ResultsTree : public Gtk::TreeView
 		  * Returns true if something was added to the tree.
 		  */
 		bool addResults(const std::string &engineName, const std::vector<DocumentInfo> &resultsList,
-			const std::string &charset);
+			const std::string &charset, bool updateHistory);
 
 		/// Sets how results are grouped.
 		void setGroupMode(GroupByMode groupMode);
@@ -79,7 +79,7 @@ class ResultsTree : public Gtk::TreeView
 		void setSelectionState(bool viewed);
 
 		/// Updates a result's properties.
-		void updateResult(const DocumentInfo &result);
+		bool updateResult(const DocumentInfo &result);
 
 		/**
 		  * Deletes the current selection.
@@ -100,24 +100,24 @@ class ResultsTree : public Gtk::TreeView
 		void clear(void);
 
 		/// Shows or hides the extract field.
-		void showExtract(bool show = true);
+		void showExtract(bool showExtract = true);
 
 		/// Exports results to a file.
 		void exportResults(const std::string &fileName, bool csvFormat);
 
 		/// Returns the changed selection signal.
-		SigC::Signal1<void, Glib::ustring>& getSelectionChangedSignal(void);
+		sigc::signal1<void, Glib::ustring>& getSelectionChangedSignal(void);
 
 		/// Returns the double-click signal.
-		SigC::Signal0<void>& getDoubleClickSignal(void);
+		sigc::signal0<void>& getDoubleClickSignal(void);
 
 	protected:
 		Glib::ustring m_treeName;
 		Gtk::Menu *m_pPopupMenu;
 		Gtk::ScrolledWindow *m_pResultsScrolledwindow;
 		Glib::RefPtr<Gtk::TreeStore> m_refStore;
-		SigC::Signal1<void, Glib::ustring> m_signalSelectionChanged;
-		SigC::Signal0<void> m_signalDoubleClick;
+		sigc::signal1<void, Glib::ustring> m_signalSelectionChanged;
+		sigc::signal0<void> m_signalDoubleClick;
 		PinotSettings &m_settings;
 		Glib::RefPtr<Gdk::Pixbuf> m_indexedIconPixbuf;
 		Glib::RefPtr<Gdk::Pixbuf> m_viewededIconPixbuf;
@@ -148,7 +148,7 @@ class ResultsTree : public Gtk::TreeView
 		void onSelectionChanged(void);
 
 		bool onSelectionSelect(const Glib::RefPtr<Gtk::TreeModel>& model,
-			const Gtk::TreeModel::Path& path, bool path_currently_selected);
+			const Gtk::TreeModel::Path& node_path, bool path_currently_selected);
 
 		/// Handles GTK style changes.
 		void onStyleChanged(const Glib::RefPtr<Gtk::Style> &previous_style);
@@ -159,8 +159,8 @@ class ResultsTree : public Gtk::TreeView
 
 		/// Adds a new row in the results tree.
 		bool appendResult(const Glib::ustring &text, const Glib::ustring &url,
-			const Glib::ustring &type, int score, int rankDiff,
-			bool isIndexed, unsigned int docId, const Glib::ustring &timestamp,
+			int score, int rankDiff, bool isIndexed, bool wasViewed,
+			unsigned int docId, const Glib::ustring &timestamp, const Glib::ustring &serial,
 			unsigned int engineId, unsigned int indexId,
 			Gtk::TreeModel::iterator &newRowIter,
 			const Gtk::TreeModel::iterator &parentIter,
@@ -171,10 +171,9 @@ class ResultsTree : public Gtk::TreeView
 
 		/// Updates a row.
 		void updateRow(Gtk::TreeModel::Row &row, const Glib::ustring &text,
-			const Glib::ustring &url, const Glib::ustring &type,
-			int score, unsigned int engineId, unsigned int indexId,
-			unsigned int docId, const Glib::ustring &timestamp,
-			ResultsModelColumns::RowType type, bool indexed, bool viewed, int rankDiff);
+			const Glib::ustring &url, int score, unsigned int engineId, unsigned int indexId,
+			unsigned int docId, const Glib::ustring &timestamp, const Glib::ustring &serial,
+			ResultsModelColumns::RowType resultType, bool indexed, bool viewed, int rankDiff);
 
 		/// Retrieves the extract to show for the given row.
 		Glib::ustring findResultsExtract(const Gtk::TreeModel::Row &row);
