@@ -28,26 +28,20 @@
 #include <exception>
 #include <iostream>
 #include <fstream>
-#include <sigc++/class_slot.h>
-#include <sigc++/compatibility.h>
-#include <sigc++/slot.h>
 #include <glibmm/miscutils.h>
 
-#include "Languages.h"
+#include "config.h"
+#include "NLS.h"
 #include "MIMEScanner.h"
 #include "TimeConverter.h"
 #include "Timer.h"
 #include "Url.h"
-#include "DBusXapianIndex.h"
-#include "XapianIndex.h"
-#include "XapianEngine.h"
-#include "config.h"
-#include "NLS.h"
+#include "DBusIndex.h"
+#include "ModuleFactory.h"
 #include "DaemonState.h"
 #include "PinotSettings.h"
 #include "ServerThreads.h"
 
-using namespace SigC;
 using namespace Glib;
 using namespace std;
 
@@ -108,250 +102,6 @@ static bool loadXMLDescription(void)
 	return readFile;
 }
 
-MonitorThread::MonitorThread(MonitorInterface *pMonitor, MonitorHandler *pHandler) :
-	WorkerThread(),
-	m_ctrlReadPipe(-1),
-	m_ctrlWritePipe(-1),
-	m_pMonitor(pMonitor),
-	m_pHandler(pHandler)
-{
-	int pipeFds[2];
-
-	if (pipe(pipeFds) == 0)
-	{
-		// This pipe will allow to stop select()
-		m_ctrlReadPipe = pipeFds[0];
-		m_ctrlWritePipe = pipeFds[1];
-	}
-}
-
-MonitorThread::~MonitorThread()
-{
-	close(m_ctrlReadPipe);
-	close(m_ctrlWritePipe);
-}
-
-string MonitorThread::getType(void) const
-{
-	return "MonitorThread";
-}
-
-bool MonitorThread::stop(void)
-{
-	// Disconnect the signal
-	Signal3<void, const DocumentInfo&, const std::string&, bool>::slot_list_type slotsList = m_signalDirectoryFound.slots();
-	Signal3<void, const DocumentInfo&, const std::string&, bool>::slot_list_type::iterator slotIter = slotsList.begin();
-	if (slotIter != slotsList.end())
-	{
-		if (slotIter->empty() == false)
-		{
-			slotIter->block();
-			slotIter->disconnect();
-		}
-	}
-	m_done = true;
-	write(m_ctrlWritePipe, "X", 1);
-
-	return true;
-}
-
-Signal3<void, const DocumentInfo&, const std::string&, bool>& MonitorThread::getDirectoryFoundSignal(void)
-{
-	return m_signalDirectoryFound;
-}
-
-void MonitorThread::processEvents(void)
-{
-	CrawlHistory history(PinotSettings::getInstance().getHistoryDatabaseName());
-	queue<MonitorEvent> events;
-
-#ifdef DEBUG
-	cout << "MonitorThread::processEvents: checking for events" << endl;
-#endif
-	if ((m_pMonitor == NULL) ||
-		(m_pMonitor->retrievePendingEvents(events) == false))
-	{
-#ifdef DEBUG
-		cout << "MonitorThread::processEvents: failed to retrieve pending events" << endl;
-#endif
-		return;
-	}
-#ifdef DEBUG
-	cout << "MonitorThread::processEvents: retrieved " << events.size() << " events" << endl;
-#endif
-
-	while ((events.empty() == false) &&
-		(m_done == false))
-	{
-		MonitorEvent &event = events.front();
-
-		if ((event.m_location.empty() == true) ||
-			(event.m_type == MonitorEvent::UNKNOWN))
-		{
-			// Next
-			events.pop();
-			continue;
-		}
-#ifdef DEBUG
-		cout << "MonitorThread::processEvents: event " << event.m_type << " on "
-			<< event.m_location << " " << event.m_isDirectory << endl;
-#endif
-
-		// Skip dotfiles and blacklisted files
-		Url urlObj("file://" + event.m_location);
-		if ((urlObj.getFile()[0] == '.') ||
-			(PinotSettings::getInstance().isBlackListed(event.m_location) == true))
-		{
-			// Next
-			events.pop();
-			continue;
-		}
-
-		// What's the event code ?
-		if (event.m_type == MonitorEvent::EXISTS)
-		{
-			if (event.m_isDirectory == false)
-			{
-				m_pHandler->fileExists(event.m_location);
-			}
-		}
-		else if (event.m_type == MonitorEvent::CREATED)
-		{
-			if (event.m_isDirectory == false)
-			{
-				m_pHandler->fileCreated(event.m_location);
-			}
-			else
-			{
-				DocumentInfo docInfo("", string("file://") + event.m_location, "", "");
-
-				// Report this directory so that it is crawled
-				m_signalDirectoryFound(docInfo, "", true);
-			}
-		}
-		else if (event.m_type == MonitorEvent::WRITE_CLOSED)
-		{
-			if (event.m_isDirectory == false)
-			{
-				CrawlHistory::CrawlStatus status = CrawlHistory::UNKNOWN;
-				struct stat fileStat;
-				time_t itemDate;
-
-				if (history.hasItem("file://" + event.m_location, status, itemDate) == true)
-				{
-					// Was the file actually modified ?
-					if ((stat(event.m_location.c_str(), &fileStat) == 0) &&
-						(itemDate < fileStat.st_mtime))
-					{
-						m_pHandler->fileModified(event.m_location);
-					}
-#ifdef DEBUG
-					else cout << "MonitorThread::processEvents: file wasn't modified" << endl;
-#endif
-				}
-#ifdef DEBUG
-				else cout << "MonitorThread::processEvents: file wasn't crawled" << endl;
-#endif
-			}
-		}
-		else if (event.m_type == MonitorEvent::MOVED)
-		{
-			if (event.m_isDirectory == false)
-			{
-				m_pHandler->fileMoved(event.m_location, event.m_previousLocation);
-			}
-			else
-			{
-				// We should receive this only if the destination directory is monitored too
-				m_pHandler->directoryMoved(event.m_location, event.m_previousLocation);
-			}
-		}
-		else if (event.m_type == MonitorEvent::DELETED)
-		{
-			if (event.m_isDirectory == false)
-			{
-				m_pHandler->fileDeleted(event.m_location);
-			}
-			else
-			{
-				// The monitor should have stopped monitoring this
-				// In practice, events for the files in this directory will already have been received 
-				m_pHandler->directoryDeleted(event.m_location);
-			}
-		}
-
-		// Next
-		events.pop();
-	}
-}
-
-void MonitorThread::doWork(void)
-{
-	if ((m_pHandler == NULL) ||
-		(m_pMonitor == NULL))
-	{
-		m_status = _("No monitoring handler");
-		return;
-	}
-
-	// Initialize the handler
-	m_pHandler->initialize();
-
-	// Get the list of files to monitor
-	const set<string> &fileNames = m_pHandler->getFileNames();
-	for (set<string>::const_iterator fileIter = fileNames.begin();
-		fileIter != fileNames.end(); ++fileIter)
-	{
-		m_pMonitor->addLocation(*fileIter, false);
-	}
-	// Directories, if any, are set elsewhere
-	// In the case of OnDiskHandler, they are set by DirectoryScannerThread
-
-	// There might already be events that need processing
-	processEvents();
-
-	// Wait for something to happen
-	while (m_done == false)
-	{
-		struct timeval selectTimeout;
-		fd_set listenSet;
-
-		selectTimeout.tv_sec = 60;
-		selectTimeout.tv_usec = 0;
-
-		FD_ZERO(&listenSet);
-		if (m_ctrlReadPipe >= 0)
-		{
-			FD_SET(m_ctrlReadPipe, &listenSet);
-		}
-
-		m_pHandler->flushIndex();
-
-		// The file descriptor may change over time
-		int monitorFd = m_pMonitor->getFileDescriptor();
-		FD_SET(monitorFd, &listenSet);
-		if (monitorFd < 0)
-		{
-			m_status = _("Couldn't initialize file monitor");
-			return;
-		}
-
-		int fdCount = select(max(monitorFd, m_ctrlReadPipe) + 1, &listenSet, NULL, NULL, &selectTimeout);
-		if ((fdCount < 0) &&
-			(errno != EINTR))
-		{
-#ifdef DEBUG
-			cout << "MonitorThread::doWork: select() failed" << endl;
-#endif
-			break;
-		}
-		else if (FD_ISSET(monitorFd, &listenSet))
-		{
-			processEvents();
-		}
-	}
-}
-
 DirectoryScannerThread::DirectoryScannerThread(const string &dirName, bool isSource,
 	bool fullScan, MonitorInterface *pMonitor, MonitorHandler *pHandler,
 	unsigned int maxLevel, bool followSymLinks) :
@@ -420,8 +170,8 @@ string DirectoryScannerThread::getDirectory(void) const
 bool DirectoryScannerThread::stop(void)
 {
 	// Disconnect the signal
-	Signal3<void, const DocumentInfo&, const std::string&, bool>::slot_list_type slotsList = m_signalFileFound.slots();
-	Signal3<void, const DocumentInfo&, const std::string&, bool>::slot_list_type::iterator slotIter = slotsList.begin();
+	sigc::signal3<void, const DocumentInfo&, const string&, bool>::slot_list_type slotsList = m_signalFileFound.slots();
+	sigc::signal3<void, const DocumentInfo&, const string&, bool>::slot_list_type::iterator slotIter = slotsList.begin();
 	if (slotIter != slotsList.end())
 	{
 		if (slotIter->empty() == false)
@@ -435,7 +185,7 @@ bool DirectoryScannerThread::stop(void)
 	return true;
 }
 
-Signal3<void, const DocumentInfo&, const std::string&, bool>& DirectoryScannerThread::getFileFoundSignal(void)
+sigc::signal3<void, const DocumentInfo&, const string&, bool>& DirectoryScannerThread::getFileFoundSignal(void)
 {
 	return m_signalFileFound;
 }
@@ -473,8 +223,7 @@ void DirectoryScannerThread::foundFile(const DocumentInfo &docInfo)
 	}
 
 	// This identifies the source
-	snprintf(labelStr, 64, "SOURCE%u", m_sourceId);
-
+	snprintf(labelStr, 64, "X-SOURCE%u", m_sourceId);
 	m_signalFileFound(docInfo, labelStr, false);
 }
 
@@ -484,7 +233,7 @@ bool DirectoryScannerThread::scanEntry(const string &entryName, CrawlHistory &hi
 	string location("file://" + entryName);
 	time_t itemDate;
 	struct stat fileStat;
-	int statSuccess = 0;
+	int entryStatus = 0;
 	bool scanSuccess = true, reportFile = false, itemExists = false;
 
 	if (entryName.empty() == true)
@@ -505,25 +254,26 @@ bool DirectoryScannerThread::scanEntry(const string &entryName, CrawlHistory &hi
 		return false;
 	}
 
+	// Is this item in the database already ?
+	itemExists = history.hasItem(location, status, itemDate);
+
 	if (m_followSymLinks == false)
 	{
-		statSuccess = lstat(entryName.c_str(), &fileStat);
+		entryStatus = lstat(entryName.c_str(), &fileStat);
 	}
 	else
 	{
 		// Stat the files pointed to by symlinks
-		statSuccess = stat(entryName.c_str(), &fileStat);
+		entryStatus = stat(entryName.c_str(), &fileStat);
 	}
 
-	// Is this item in the database already ?
-	itemExists = history.hasItem(location, status, itemDate);
-
-	if (statSuccess == -1)
+	if (entryStatus == -1)
 	{
-#ifdef DEBUG
-		cout << "DirectoryScannerThread::scanEntry: stat failed with error " << errno << " " << strerror(errno) << endl;
-#endif
+		entryStatus = errno;
 		scanSuccess = false;
+#ifdef DEBUG
+		cout << "DirectoryScannerThread::scanEntry: stat failed with error " << entryStatus << endl;
+#endif
 	}
 	// Is it a file or a directory ?
 	else if (S_ISLNK(fileStat.st_mode))
@@ -563,7 +313,12 @@ bool DirectoryScannerThread::scanEntry(const string &entryName, CrawlHistory &hi
 				if (m_pMonitor != NULL)
 				{
 					// Monitor first so that we don't miss events
-					m_pMonitor->addLocation(entryName, true);
+					// If monitoring is not possible, record the first case
+					if ((m_pMonitor->addLocation(entryName, true) == false) &&
+						(entryStatus != MONITORING_FAILED))
+					{
+						entryStatus = MONITORING_FAILED;
+					}
 				}
 
 				// Iterate through this directory's entries
@@ -609,10 +364,11 @@ bool DirectoryScannerThread::scanEntry(const string &entryName, CrawlHistory &hi
 			}
 			else
 			{
-#ifdef DEBUG
-				cout << "DirectoryScannerThread::scanEntry: opendir failed with error " << errno << " " << strerror(errno) << endl;
-#endif
+				entryStatus = errno;
 				scanSuccess = false;
+#ifdef DEBUG
+				cout << "DirectoryScannerThread::scanEntry: opendir failed with error " << entryStatus << endl;
+#endif
 			}
 		}
 	}
@@ -621,30 +377,18 @@ bool DirectoryScannerThread::scanEntry(const string &entryName, CrawlHistory &hi
 #ifdef DEBUG
 		cout << "DirectoryScannerThread::scanEntry: unknown entry type" << endl;
 #endif
+		entryStatus = ENOENT;
 		scanSuccess = false;
 	}
 
-	// Did an error occur ?
-	if (scanSuccess == false)
-	{
-		time_t timeNow = time(NULL);
-
-		// Record this error
-		if (itemExists == false)
-		{
-			history.insertItem(location, CrawlHistory::ERROR, m_sourceId, timeNow);
-		}
-		else
-		{
-			history.updateItem(location, CrawlHistory::ERROR, timeNow);
-		}
-	}
-	else if (reportFile == true)
+	// If a major error occured, this won't be true
+	if (reportFile == true)
 	{
 		if (itemExists == false)
 		{
 			// Record it
 			history.insertItem(location, CrawlHistory::CRAWLED, m_sourceId, fileStat.st_mtime);
+			itemExists = true;
 #ifdef DEBUG
 			cout << "DirectoryScannerThread::scanEntry: reporting new file " << entryName << endl;
 #endif
@@ -674,10 +418,8 @@ bool DirectoryScannerThread::scanEntry(const string &entryName, CrawlHistory &hi
 
 		if (reportFile == true)
 		{
-			DocumentInfo docInfo;
+			DocumentInfo docInfo("", location, "", "");
 
-			docInfo.setLocation(location);
-			docInfo.setTitle(urlObj.getFile());
 			if (S_ISDIR(fileStat.st_mode))
 			{
 				docInfo.setType("x-directory/normal");
@@ -694,6 +436,22 @@ bool DirectoryScannerThread::scanEntry(const string &entryName, CrawlHistory &hi
 		}
 	}
 
+	// Did an error occur ?
+	if (entryStatus != 0)
+	{
+		time_t timeNow = time(NULL);
+
+		// Record this error
+		if (itemExists == false)
+		{
+			history.insertItem(location, CrawlHistory::ERROR, m_sourceId, timeNow, entryStatus);
+		}
+		else
+		{
+			history.updateItem(location, CrawlHistory::ERROR, timeNow, entryStatus);
+		}
+	}
+
 	return scanSuccess;
 }
 
@@ -721,9 +479,8 @@ void DirectoryScannerThread::doWork(void)
 
 	if (scanEntry(m_dirName, history) == false)
 	{
-		m_status = _("Couldn't open directory");
-		m_status += " ";
-		m_status += m_dirName;
+		m_errorNum = OPENDIR_FAILED;
+		m_errorParam = m_dirName;
 	}
 	flushUpdates(history);
 
@@ -793,12 +550,6 @@ string DBusServletThread::getType(void) const
 	return "DBusServletThread";
 }
 
-bool DBusServletThread::stop(void)
-{
-	m_done = true;
-	return true;
-}
-
 DBusConnection *DBusServletThread::getConnection(void) const
 {
 	return m_pConnection;
@@ -816,17 +567,42 @@ bool DBusServletThread::mustQuit(void) const
 
 bool DBusServletThread::runQuery(QueryProperties &queryProps, vector<string> &docIds)
 {
-	XapianEngine engine(PinotSettings::getInstance().m_daemonIndexLocation);
+	PinotSettings &settings = PinotSettings::getInstance();
 
-	// Run the query
-	if (engine.runQuery(queryProps) == false)
+	docIds.clear();
+
+	SearchEngineInterface *pEngine = ModuleFactory::getSearchEngine(settings.m_defaultBackend,
+		settings.m_daemonIndexLocation);
+	if (pEngine == NULL)
 	{
 		return false;
 	}
 
-	docIds.clear();
+	// Run the query
+	pEngine->setDefaultOperator(SearchEngineInterface::DEFAULT_OP_AND);
+	if (pEngine->runQuery(queryProps) == false)
+	{
+		delete pEngine;
+
+		return false;
+	}
+
+	const vector<DocumentInfo> &resultsList = pEngine->getResults();
+	if (resultsList.empty() == true)
+	{
+#ifdef DEBUG
+		cout << "DBusServletThread::runQuery: trying again" << endl;
+#endif
+		// Try again, this time with OR as default operator
+		pEngine->setDefaultOperator(SearchEngineInterface::DEFAULT_OP_OR);
+		if (pEngine->runQuery(queryProps) == false)
+		{
+			delete pEngine;
+
+			return false;
+		}
+	}
 
-	const vector<DocumentInfo> &resultsList = engine.getResults();
 	for (vector<DocumentInfo>::const_iterator resultIter = resultsList.begin();
 		resultIter != resultsList.end(); ++resultIter)
 	{
@@ -843,24 +619,35 @@ bool DBusServletThread::runQuery(QueryProperties &queryProps, vector<string> &do
 		}
 	}
 
+	delete pEngine;
+
 	return true;
 }
 
 void DBusServletThread::doWork(void)
 {
-	XapianIndex index(PinotSettings::getInstance().m_daemonIndexLocation);
+	PinotSettings &settings = PinotSettings::getInstance();
+	IndexInterface *pIndex = settings.getIndex(settings.m_daemonIndexLocation);
 	DBusError error;
-	bool processedMessage = true, flushIndex = false;
+	bool processedMessage = true, updateLabelsCache = false, flushIndex = false;
 
 	if ((m_pServer == NULL) ||
 		(m_pConnection == NULL) ||
-		(m_pRequest == NULL))
+		(m_pRequest == NULL) ||
+		(pIndex == NULL))
 	{
 		return;
 	}
 
 	dbus_error_init(&error);
 
+	// Access the settings' labels list directly
+	set<string> &labelsCache = settings.m_labels;
+	if (labelsCache.empty() == true)
+	{
+		pIndex->getLabels(labelsCache);
+	}
+
 #ifdef DEBUG
 	const char *pSender = dbus_message_get_sender(m_pRequest);
 	if (pSender != NULL)
@@ -875,9 +662,9 @@ void DBusServletThread::doWork(void)
 
 	if (dbus_message_is_method_call(m_pRequest, "de.berlios.Pinot", "GetStatistics") == TRUE)
 	{
-		CrawlHistory history(PinotSettings::getInstance().getHistoryDatabaseName());
+		CrawlHistory history(settings.getHistoryDatabaseName());
 		unsigned int crawledFilesCount = history.getItemsCount(CrawlHistory::CRAWLED);
-		unsigned int docsCount = index.getDocumentsCount();
+		unsigned int docsCount = pIndex->getDocumentsCount();
 
 #ifdef DEBUG
 		cout << "DBusServletThread::doWork: received GetStatistics" << endl;
@@ -892,6 +679,144 @@ void DBusServletThread::doWork(void)
 				DBUS_TYPE_INVALID);
 		}
 	}
+	else if (dbus_message_is_method_call(m_pRequest, "de.berlios.Pinot", "Reload") == TRUE)
+	{
+		if (dbus_message_get_args(m_pRequest, &error,
+			DBUS_TYPE_INVALID) == TRUE)
+		{
+			gboolean reloading = TRUE;
+
+#ifdef DEBUG
+			cout << "DBusServletThread::doWork: received Reload" << endl;
+#endif
+			m_pServer->reload();
+
+			// Prepare the reply
+			m_pReply = newDBusReply(m_pRequest);
+			if (m_pReply != NULL)
+			{
+				dbus_message_append_args(m_pReply,
+					DBUS_TYPE_BOOLEAN, &reloading,
+					DBUS_TYPE_INVALID);
+			}
+		}
+	}
+	else if (dbus_message_is_method_call(m_pRequest, "de.berlios.Pinot", "Stop") == TRUE)
+	{
+		if (dbus_message_get_args(m_pRequest, &error,
+			DBUS_TYPE_INVALID) == TRUE)
+		{
+			int exitStatus = EXIT_SUCCESS;
+
+#ifdef DEBUG
+			cout << "DBusServletThread::doWork: received Stop" << endl;
+#endif
+			// Prepare the reply
+			m_pReply = newDBusReply(m_pRequest);
+			if (m_pReply != NULL)
+			{
+				dbus_message_append_args(m_pReply,
+					DBUS_TYPE_INT32, &exitStatus,
+					DBUS_TYPE_INVALID);
+			}
+
+			m_mustQuit = true;
+		}
+	}
+	else if (dbus_message_is_method_call(m_pRequest, "de.berlios.Pinot", "HasDocument") == TRUE)
+	{
+		char *pUrl = NULL;
+		unsigned int docId = 0;
+
+		if (dbus_message_get_args(m_pRequest, &error,
+			DBUS_TYPE_STRING, &pUrl,
+			DBUS_TYPE_INVALID) == TRUE)
+		{
+#ifdef DEBUG
+			cout << "DBusServletThread::doWork: received HasDocument " << pUrl << endl;
+#endif
+			if (pUrl != NULL)
+			{
+				string url(pUrl);
+
+				// Check the index
+				docId = pIndex->hasDocument(url);
+			}
+
+			// Prepare the reply
+			m_pReply = newDBusReply(m_pRequest);
+			if (m_pReply != NULL)
+			{
+				dbus_message_append_args(m_pReply,
+					DBUS_TYPE_UINT32, &docId,
+					DBUS_TYPE_INVALID);
+			}
+		}
+	}
+	else if (dbus_message_is_method_call(m_pRequest, "de.berlios.Pinot", "GetLabels") == TRUE)
+	{
+#ifdef DEBUG
+		cout << "DBusServletThread::doWork: received GetLabels" << endl;
+#endif
+		// This method doesn't take any argument
+		m_pArray = g_ptr_array_new();
+
+		for (set<string>::const_iterator labelIter = labelsCache.begin();
+			labelIter != labelsCache.end(); ++labelIter)
+		{
+			string labelName(*labelIter);
+
+			g_ptr_array_add(m_pArray, const_cast<char*>(labelName.c_str()));
+#ifdef DEBUG
+			cout << "DBusServletThread::doWork: adding label " << m_pArray->len << " " << labelName << endl;
+#endif
+		}
+
+		// Prepare the reply
+		m_pReply = newDBusReply(m_pRequest);
+		if (m_pReply != NULL)
+		{
+			dbus_message_append_args(m_pReply,
+				DBUS_TYPE_ARRAY, DBUS_TYPE_STRING, &m_pArray->pdata, m_pArray->len,
+				DBUS_TYPE_INVALID);
+		}
+	}
+	else if (dbus_message_is_method_call(m_pRequest, "de.berlios.Pinot", "AddLabel") == TRUE)
+	{
+		char *pLabel = NULL;
+
+		if (dbus_message_get_args(m_pRequest, &error,
+			DBUS_TYPE_STRING, &pLabel,
+			DBUS_TYPE_INVALID) == TRUE)
+		{
+#ifdef DEBUG
+			cout << "DBusServletThread::doWork: received AddLabel " << pLabel << endl;
+#endif
+			if (pLabel != NULL)
+			{
+				string labelName(pLabel);
+
+				// Add the label
+				flushIndex = pIndex->addLabel(labelName);
+				// Is this a known label ?
+				if (labelsCache.find(labelName) == labelsCache.end())
+				{
+					// No, it isn't but that's okay
+					labelsCache.insert(labelName);
+					updateLabelsCache = true;
+				}
+			}
+
+			// Prepare the reply
+			m_pReply = newDBusReply(m_pRequest);
+			if (m_pReply != NULL)
+			{
+				dbus_message_append_args(m_pReply,
+					DBUS_TYPE_STRING, &pLabel,
+					DBUS_TYPE_INVALID);
+			}
+		}
+	}
 	else if (dbus_message_is_method_call(m_pRequest, "de.berlios.Pinot", "RenameLabel") == TRUE)
 	{
 		char *pOldLabel = NULL;
@@ -905,8 +830,21 @@ void DBusServletThread::doWork(void)
 #ifdef DEBUG
 			cout << "DBusServletThread::doWork: received RenameLabel " << pOldLabel << ", " << pNewLabel << endl;
 #endif
-			// Rename the label
-			flushIndex = index.renameLabel(pOldLabel, pNewLabel);
+
+			if ((pOldLabel != NULL) &&
+				(pNewLabel != NULL))
+			{
+				// Rename the label
+				flushIndex = pIndex->renameLabel(pOldLabel, pNewLabel);
+				// Update the labels list
+				set<string>::const_iterator oldLabelIter = labelsCache.find(pOldLabel);
+				if (oldLabelIter != labelsCache.end())
+				{
+					labelsCache.erase(oldLabelIter);
+					labelsCache.insert(pNewLabel);
+					updateLabelsCache = true;
+				}
+			}
 
 			// Prepare the reply
 			m_pReply = newDBusReply(m_pRequest);
@@ -929,8 +867,18 @@ void DBusServletThread::doWork(void)
 #ifdef DEBUG
 			cout << "DBusServletThread::doWork: received DeleteLabel " << pLabel << endl;
 #endif
-			// Delete the label
-			flushIndex = index.deleteLabel(pLabel);
+			if (pLabel != NULL)
+			{
+				// Delete the label
+				flushIndex = pIndex->deleteLabel(pLabel);
+				// Update the labels list
+				set<string>::const_iterator labelIter = labelsCache.find(pLabel);
+				if (labelIter != labelsCache.end())
+				{
+					labelsCache.erase(labelIter);
+					updateLabelsCache = true;
+				}
+			}
 
 			// Prepare the reply
 			m_pReply = newDBusReply(m_pRequest);
@@ -955,7 +903,7 @@ void DBusServletThread::doWork(void)
 #ifdef DEBUG
 			cout << "DBusServletThread::doWork: received GetDocumentLabels " << docId << endl;
 #endif
-			if (index.getDocumentLabels(docId, labels) == true)
+			if (pIndex->getDocumentLabels(docId, labels) == true)
 			{
 				m_pArray = g_ptr_array_new();
 
@@ -1008,14 +956,23 @@ void DBusServletThread::doWork(void)
 				{
 					break;
 				}
-				labels.insert(ppLabels[labelIndex]);
+
+				string labelName(ppLabels[labelIndex]);
+				labels.insert(labelName);
+				// Is this a known label ?
+				if (labelsCache.find(labelName) == labelsCache.end())
+				{
+					// No, it isn't but that's okay
+					labelsCache.insert(labelName);
+					updateLabelsCache = true;
+				}
 			}
 #ifdef DEBUG
 			cout << "DBusServletThread::doWork: received SetDocumentLabels on ID " << docId
 				<< ", " << labelsCount << " labels" << ", " << resetLabels << endl;
 #endif
 			// Set labels
-			flushIndex = index.setDocumentLabels(docId, labels, ((resetLabels == TRUE) ? true : false));
+			flushIndex = pIndex->setDocumentLabels(docId, labels, ((resetLabels == TRUE) ? true : false));
 
 			// Free container types
 			g_strfreev(ppLabels);
@@ -1061,14 +1018,27 @@ void DBusServletThread::doWork(void)
 				{
 					break;
 				}
-				labels.insert(ppLabels[labelIndex]);
+
+				string labelName(ppLabels[labelIndex]);
+				labels.insert(labelName);
+				// Is this a known label ?
+				if (labelsCache.find(labelName) == labelsCache.end())
+				{
+					// No, it isn't but that's okay
+					labelsCache.insert(labelName);
+					updateLabelsCache = true;
+				}
 			}
 #ifdef DEBUG
 			cout << "DBusServletThread::doWork: received SetDocumentsLabels on " << docIds.size()
 				<< " IDs, " << labelsCount << " labels" << ", " << resetLabels << endl;
 #endif
 			// Set labels
-			flushIndex = index.setDocumentsLabels(docIds, labels, ((resetLabels == TRUE) ? true : false));
+			if (pIndex->setDocumentsLabels(docIds, labels, ((resetLabels == TRUE) ? true : false)) == true)
+			{
+				resetLabels = TRUE;
+				flushIndex = true;
+			}
 
 			// Free container types
 			g_strfreev(ppDocIds);
@@ -1079,7 +1049,7 @@ void DBusServletThread::doWork(void)
 			if (m_pReply != NULL)
 			{
 				dbus_message_append_args(m_pReply,
-					DBUS_TYPE_BOOLEAN, &flushIndex,
+					DBUS_TYPE_BOOLEAN, &resetLabels,
 					DBUS_TYPE_INVALID);
 			}
 		}
@@ -1097,7 +1067,7 @@ void DBusServletThread::doWork(void)
 #ifdef DEBUG
 			cout << "DBusServletThread::doWork: received GetDocumentInfo on " << docId << endl;
 #endif
-			if (index.getDocumentInfo(docId, docInfo) == true)
+			if (pIndex->getDocumentInfo(docId, docInfo) == true)
 			{
 				// Prepare the reply
 				m_pReply = newDBusReply(m_pRequest);
@@ -1106,7 +1076,7 @@ void DBusServletThread::doWork(void)
 					DBusMessageIter iter;
 
 					dbus_message_iter_init_append(m_pReply, &iter);
-					if (DBusXapianIndex::documentInfoToDBus(&iter, 0, docInfo) == false)
+					if (DBusIndex::documentInfoToDBus(&iter, 0, docInfo) == false)
 					{
 						dbus_message_unref(m_pReply);
 						m_pReply = dbus_message_new_error(m_pRequest,
@@ -1130,7 +1100,7 @@ void DBusServletThread::doWork(void)
 		unsigned int docId = 0;
 
 		dbus_message_iter_init(m_pRequest, &iter);
-		if (DBusXapianIndex::documentInfoFromDBus(&iter, docId, docInfo) == false)
+		if (DBusIndex::documentInfoFromDBus(&iter, docId, docInfo) == false)
 		{
 			m_pReply = dbus_message_new_error(m_pRequest,
 				"de.berlios.Pinot.SetDocumentInfo",
@@ -1143,7 +1113,7 @@ void DBusServletThread::doWork(void)
 #endif
 
 			// Update the document info
-			flushIndex = index.updateDocumentInfo(docId, docInfo);
+			flushIndex = pIndex->updateDocumentInfo(docId, docInfo);
 
 			// Prepare the reply
 			m_pReply = newDBusReply(m_pRequest);
@@ -1212,28 +1182,6 @@ void DBusServletThread::doWork(void)
 			}
 		}
 	}
-	else if (dbus_message_is_method_call(m_pRequest, "de.berlios.Pinot", "Stop") == TRUE)
-	{
-		if (dbus_message_get_args(m_pRequest, &error,
-			DBUS_TYPE_INVALID) == TRUE)
-		{
-			int exitStatus = EXIT_SUCCESS;
-
-#ifdef DEBUG
-			cout << "DBusServletThread::doWork: received Stop" << endl;
-#endif
-			// Prepare the reply
-			m_pReply = newDBusReply(m_pRequest);
-			if (m_pReply != NULL)
-			{
-				dbus_message_append_args(m_pReply,
-					DBUS_TYPE_INT32, &exitStatus,
-					DBUS_TYPE_INVALID);
-			}
-
-			m_mustQuit = true;
-		}
-	}
 	else if (dbus_message_is_method_call(m_pRequest, "de.berlios.Pinot", "UpdateDocument") == TRUE)
 	{
 		unsigned int docId = 0;
@@ -1247,7 +1195,7 @@ void DBusServletThread::doWork(void)
 #ifdef DEBUG
 			cout << "DBusServletThread::doWork: received UpdateDocument " << docId << endl;
 #endif
-			if (index.getDocumentInfo(docId, docInfo) == true)
+			if (pIndex->getDocumentInfo(docId, docInfo) == true)
 			{
 				// Update document
 				m_pServer->queue_index(docInfo);
@@ -1303,10 +1251,16 @@ void DBusServletThread::doWork(void)
 
 	dbus_error_free(&error);
 
-	if (flushIndex == true)
+	// Set labels ?
+	if ((updateLabelsCache == true) &&
+		(pIndex->setLabels(labelsCache) == false))
 	{
-		// Flush now for the sake of the client application
-		index.flush();
+		// Updating failed... reset the cache
+		labelsCache.clear();
+		pIndex->getLabels(labelsCache);
+#ifdef DEBUG
+		cout << "DBusServletThread::doWork: failed to update labels" << endl;
+#endif
 	}
 
 	// Send a reply ?
@@ -1320,5 +1274,14 @@ void DBusServletThread::doWork(void)
 #endif
 		dbus_message_unref(m_pReply);
 	}
+
+	// Flush the index ?
+	if (flushIndex == true)
+	{
+		// Flush now for the sake of the client application
+		pIndex->flush();
+	}
+
+	delete pIndex;
 }
 
