@@ -23,14 +23,14 @@
 #include <vector>
 extern "C"
 {
+#if DBUS_VERSION < 1000000
 #define DBUS_API_SUBJECT_TO_CHANGE
+#endif
 #include <dbus/dbus.h>
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
 }
-#include <sigc++/object.h>
-#include <sigc++/slot.h>
-#include <sigc++/connection.h>
+#include <sigc++/sigc++.h>
 #include <glibmm/ustring.h>
 
 #include "DocumentInfo.h"
@@ -41,34 +41,6 @@ extern "C"
 #include "DaemonState.h"
 #include "WorkerThreads.h"
 
-class MonitorThread : public WorkerThread
-{
-	public:
-		MonitorThread(MonitorInterface *pMonitor, MonitorHandler *pHandler);
-		virtual ~MonitorThread();
-
-		virtual std::string getType(void) const;
-
-		virtual bool stop(void);
-
-		SigC::Signal3<void, const DocumentInfo&, const std::string&, bool>& getDirectoryFoundSignal(void);
-
-	protected:
-		int m_ctrlReadPipe;
-		int m_ctrlWritePipe;
-		MonitorInterface *m_pMonitor;
-		MonitorHandler *m_pHandler;
-		SigC::Signal3<void, const DocumentInfo&, const std::string&, bool> m_signalDirectoryFound;
-
-		void processEvents(void);
-		virtual void doWork(void);
-
-	private:
-		MonitorThread(const MonitorThread &other);
-		MonitorThread &operator=(const MonitorThread &other);
-
-};
-
 class DirectoryScannerThread : public WorkerThread
 {
 	public:
@@ -83,7 +55,7 @@ class DirectoryScannerThread : public WorkerThread
 
 		virtual bool stop(void);
 
-		SigC::Signal3<void, const DocumentInfo&, const std::string&, bool>& getFileFoundSignal(void);
+		sigc::signal3<void, const DocumentInfo&, const std::string&, bool>& getFileFoundSignal(void);
 
 	protected:
 		std::string m_dirName;
@@ -94,7 +66,7 @@ class DirectoryScannerThread : public WorkerThread
 		unsigned int m_currentLevel;
 		unsigned int m_maxLevel;
 		bool m_followSymLinks;
-		SigC::Signal3<void, const DocumentInfo&, const std::string&, bool> m_signalFileFound;
+		sigc::signal3<void, const DocumentInfo&, const std::string&, bool> m_signalFileFound;
 		std::map<std::string, time_t> m_updateCache;
 
 		void cacheUpdate(const std::string &location, time_t mTime, CrawlHistory &history);
@@ -117,8 +89,6 @@ class DBusServletThread : public WorkerThread
 
 		virtual std::string getType(void) const;
 
-		virtual bool stop(void);
-
 		DBusConnection *getConnection(void) const;
 
 		DBusMessage *getReply(void) const;
