@@ -26,15 +26,15 @@
 #include <set>
 #include <map>
 #include <pthread.h>
-#include <sigc++/object.h>
-#include <sigc++/slot.h>
-#include <sigc++/connection.h>
+#include <sigc++/sigc++.h>
 #include <glibmm/dispatcher.h>
 #include <glibmm/thread.h>
 #include <glibmm/ustring.h>
 
 #include "Document.h"
 #include "DownloaderInterface.h"
+#include "MonitorInterface.h"
+#include "MonitorHandler.h"
 #include "QueryProperties.h"
 
 class WorkerThread
@@ -43,17 +43,26 @@ class WorkerThread
 		WorkerThread();
 		virtual ~WorkerThread();
 
+		typedef enum { UNKNOWN_ERROR = 10000, INDEX_ERROR, INDEXING_FAILED, UPDATE_FAILED, UNINDEXING_FAILED, \
+			QUERY_FAILED, HISTORY_FAILED, DOWNLOAD_FAILED, MONITORING_FAILED, OPENDIR_FAILED, \
+			UNKNOWN_INDEX, UNKNOWN_ENGINE, UNSUPPORTED_TYPE, UNSUPPORTED_PROTOCOL, \
+			ROBOTS_FORBIDDEN, NO_MONITORING } ThreadError;
+
+		static std::string errorToString(int errorNum);
+
 		static Glib::Dispatcher &getDispatcher(void);
 
 		static void immediateFlush(bool doFlush);
 
+		time_t getStartTime(void) const;
+
 		void setId(unsigned int id);
 
-		unsigned int getId(void);
+		unsigned int getId(void) const;
 
 		void inBackground(void);
 
-		bool isBackground(void);
+		bool isBackground(void) const;
 
 		bool operator<(const WorkerThread &other) const;
 
@@ -61,10 +70,14 @@ class WorkerThread
 
 		virtual std::string getType(void) const = 0;
 
-		virtual bool stop(void) = 0;
+		virtual bool stop(void);
+
+		bool isStopped(void) const;
 
 		bool isDone(void) const;
 
+		int getErrorNum(void) const;
+
 		std::string getStatus(void) const;
 
 	protected:
@@ -72,10 +85,13 @@ class WorkerThread
 		static Glib::Dispatcher m_dispatcher;
 		static pthread_mutex_t m_dispatcherMutex;
 		static bool m_immediateFlush;
+		time_t m_startTime;
 		unsigned int m_id;
 		bool m_background;
+		bool m_stopped;
 		bool m_done;
-		std::string m_status;
+		int m_errorNum;
+		std::string m_errorParam;
 
 		void threadHandler(void);
 
@@ -89,7 +105,7 @@ class WorkerThread
 
 };
 
-class ThreadsManager : virtual public SigC::Object
+class ThreadsManager : virtual public sigc::trackable
 {
 	public:
 		ThreadsManager(const std::string &defaultIndexLocation,
@@ -123,7 +139,7 @@ class ThreadsManager : virtual public SigC::Object
 		void get_statistics(unsigned int &queueSize);
 
 	protected:
-		SigC::Connection m_threadsEndConnection;
+		sigc::connection m_threadsEndConnection;
 		pthread_rwlock_t m_threadsLock;
 		pthread_rwlock_t m_listsLock;
 		std::map<WorkerThread *, Glib::Thread *> m_threads;
@@ -132,15 +148,22 @@ class ThreadsManager : virtual public SigC::Object
 		unsigned int m_nextThreadId;
 		unsigned int m_backgroundThreadsCount;
 		long m_numCPUs;
-		SigC::Signal1<void, WorkerThread *> m_onThreadEndSignal;
+		sigc::signal1<void, WorkerThread *> m_onThreadEndSignal;
 		std::set<std::string> m_beingIndexed;
+		bool m_stopIndexing;
 
 		bool read_lock_threads(void);
+
 		bool write_lock_threads(void);
+
 		void unlock_threads(void);
+
 		WorkerThread *get_thread(void);
+
 		Glib::ustring index_document(const DocumentInfo &docInfo);
 
+		void clear_queues(void);
+
 	private:
 		ThreadsManager(const ThreadsManager &other);
 		ThreadsManager &operator=(const ThreadsManager &other);
@@ -184,8 +207,6 @@ class IndexBrowserThread : public ListerThread
 
 		std::string getLabelName(void) const;
 
-		virtual bool stop(void);
-
 	protected:
 		unsigned int m_maxDocsCount;
 
@@ -207,16 +228,14 @@ class QueryingThread : public ListerThread
 
 		virtual std::string getType(void) const;
 
+		bool isLive(void) const;
+
 		std::string getEngineName(void) const;
 
-		bool isListingIndex(void) const;
-
-		QueryProperties getQuery(void) const;
+		QueryProperties getQuery(bool &wasCorrected) const;
 
 		std::string getCharset(void) const;
 
-		virtual bool stop(void);
-
 	protected:
 		std::string m_engineName;
 		std::string m_engineDisplayableName;
@@ -224,6 +243,28 @@ class QueryingThread : public ListerThread
 		QueryProperties m_queryProps;
 		std::string m_resultsCharset;
 		bool m_listingIndex;
+		bool m_correctedSpelling;
+		bool m_isLive;
+
+	private:
+		QueryingThread(const QueryingThread &other);
+		QueryingThread &operator=(const QueryingThread &other);
+
+};
+
+class EngineQueryThread : public QueryingThread
+{
+	public:
+		EngineQueryThread(const std::string &engineName, const std::string &engineDisplayableName,
+			const std::string &engineOption, const QueryProperties &queryProps,
+			unsigned int startDoc = 0, bool listingIndex = false);
+		EngineQueryThread(const std::string &engineName, const std::string &engineDisplayableName,
+			const std::string &engineOption, const QueryProperties &queryProps,
+			const std::set<std::string> &limitToDocsSet, unsigned int startDoc = 0);
+		virtual ~EngineQueryThread();
+
+	protected:
+		std::set<std::string> m_limitToDocsSet;
 
 		virtual void processResults(const std::vector<DocumentInfo> &resultsList);
 
@@ -233,8 +274,26 @@ class QueryingThread : public ListerThread
 		virtual void doWork(void);
 
 	private:
-		QueryingThread(const QueryingThread &other);
-		QueryingThread &operator=(const QueryingThread &other);
+		EngineQueryThread(const EngineQueryThread &other);
+		EngineQueryThread &operator=(const EngineQueryThread &other);
+
+};
+
+class EngineHistoryThread : public QueryingThread
+{
+	public:
+		EngineHistoryThread(const std::string &engineDisplayableName,
+			const QueryProperties &queryProps, unsigned int maxDocsCount);
+		virtual ~EngineHistoryThread();
+
+	protected:
+		unsigned int m_maxDocsCount;
+
+		virtual void doWork(void);
+
+	private:
+		EngineHistoryThread(const EngineHistoryThread &other);
+		EngineHistoryThread &operator=(const EngineHistoryThread &other);
 
 };
 
@@ -242,7 +301,7 @@ class ExpandQueryThread : public WorkerThread
 {
 	public:
 		ExpandQueryThread(const QueryProperties &queryProps,
-			const std::set<std::string> &relevantDocs);
+			const std::set<std::string> &expandFromDocsSet);
 		virtual ~ExpandQueryThread();
 
 		virtual std::string getType(void) const;
@@ -251,11 +310,9 @@ class ExpandQueryThread : public WorkerThread
 
 		const std::set<std::string> &getExpandTerms(void) const;
 
-		virtual bool stop(void);
-
 	protected:
 		QueryProperties m_queryProps;
-		std::set<std::string> m_relevantDocs;
+		std::set<std::string> m_expandFromDocsSet;
 		std::set<std::string> m_expandTerms;
 
 		virtual void doWork(void);
@@ -269,24 +326,25 @@ class ExpandQueryThread : public WorkerThread
 class LabelUpdateThread : public WorkerThread
 {
 	public:
-		LabelUpdateThread(const std::set<std::string> &labelsToDelete,
+		LabelUpdateThread(const std::set<std::string> &labelsToAdd,
+			const std::set<std::string> &labelsToDelete,
 			const std::map<std::string, std::string> &labelsToRename);
-		LabelUpdateThread(const std::string &labelName,
+		LabelUpdateThread(const std::set<std::string> &labelsToAdd,
 			const std::set<unsigned int> &docsIds,
-			const std::set<unsigned int> &daemonIds);
+			const std::set<unsigned int> &daemonIds,
+			bool resetLabels);
 
 		virtual ~LabelUpdateThread();
 
 		virtual std::string getType(void) const;
 
-		virtual bool stop(void);
-
 	protected:
+		std::set<std::string> m_labelsToAdd;
 		std::set<std::string> m_labelsToDelete;
 		std::map<std::string, std::string> m_labelsToRename;
-		std::string m_labelName;
 		std::set<unsigned int> m_docsIds;
 		std::set<unsigned int> m_daemonIds;
+		bool m_resetLabels;
 
 		virtual void doWork(void);
 
@@ -308,8 +366,6 @@ class DownloadingThread : public WorkerThread
 
 		const Document *getDocument(void) const;
 
-		virtual bool stop(void);
-
 	protected:
 		DocumentInfo m_docInfo;
 		Document *m_pDoc;
@@ -340,8 +396,6 @@ class IndexingThread : public DownloadingThread
 
 		bool isNewDocument(void) const;
 
-		virtual bool stop(void);
-
 	protected:
 		DocumentInfo m_docInfo;
 		unsigned int m_docId;
@@ -370,8 +424,6 @@ class UnindexingThread : public WorkerThread
 
 		unsigned int getDocumentsCount(void) const;
 
-		virtual bool stop(void);
-
 	protected:
 		std::set<unsigned int> m_docIdList;
 		std::set<std::string> m_labelNames;
@@ -390,22 +442,23 @@ class UpdateDocumentThread : public WorkerThread
 {
 	public:
 		// Update a document's properties
-		UpdateDocumentThread(const std::string &indexName,
-			unsigned int docId, const DocumentInfo &docInfo);
+		UpdateDocumentThread(const std::string &indexName, unsigned int docId,
+			const DocumentInfo &docInfo, bool updateLabels);
 		virtual ~UpdateDocumentThread();
 
 		virtual std::string getType(void) const;
 
+		std::string getIndexName(void) const;
+
 		unsigned int getDocumentID(void) const;
 
 		const DocumentInfo &getDocumentInfo(void) const;
 
-		virtual bool stop(void);
-
 	protected:
 		std::string m_indexName;
 		unsigned int m_docId;
 		DocumentInfo m_docInfo;
+		bool m_updateLabels;
 
 		virtual void doWork(void);
 
@@ -424,8 +477,6 @@ class StartDaemonThread : public WorkerThread
 
 		virtual std::string getType(void) const;
 
-		virtual bool stop(void);
-
 	protected:
 		virtual void doWork(void);
 
@@ -435,4 +486,31 @@ class StartDaemonThread : public WorkerThread
 
 };
 
+class MonitorThread : public WorkerThread
+{
+	public:
+		MonitorThread(MonitorInterface *pMonitor, MonitorHandler *pHandler,
+			bool checkHistory = true);
+		virtual ~MonitorThread();
+
+		virtual std::string getType(void) const;
+
+		virtual bool stop(void);
+
+	protected:
+		int m_ctrlReadPipe;
+		int m_ctrlWritePipe;
+		MonitorInterface *m_pMonitor;
+		MonitorHandler *m_pHandler;
+		bool m_checkHistory;
+
+		void processEvents(void);
+		virtual void doWork(void);
+
+	private:
+		MonitorThread(const MonitorThread &other);
+		MonitorThread &operator=(const MonitorThread &other);
+
+};
+
 #endif // _WORKERTHREADS_HH
