@@ -26,6 +26,7 @@
 #include <unistd.h>
 #include <iostream>
 #include <fstream>
+#include <sigc++/sigc++.h>
 #include <glibmm.h>
 #include <glibmm/thread.h>
 #include <glibmm/ustring.h>
@@ -34,20 +35,17 @@
 #include <glibmm/object.h>
 #include <glibmm/main.h>
 
+#include "config.h"
+#include "NLS.h"
 #include "FilterFactory.h"
 #include "Languages.h"
 #include "MIMEScanner.h"
-#include "XapianDatabase.h"
-#include "XapianDatabaseFactory.h"
+#include "ModuleFactory.h"
 #include "ActionQueue.h"
 #include "CrawlHistory.h"
 #include "QueryHistory.h"
 #include "ViewHistory.h"
 #include "DownloaderInterface.h"
-#include "XapianIndex.h"
-#include "XapianEngine.h"
-#include "config.h"
-#include "NLS.h"
 #include "DaemonState.h"
 #include "PinotSettings.h"
 #include "ServerThreads.h"
@@ -62,6 +60,7 @@ static struct option g_longOptions[] = {
 	{"fullscan", 1, 0, 'f'},
 	{"help", 0, 0, 'h'},
 	{"priority", 1, 0, 'p'},
+	{"reindex", 0, 0, 'r'},
 	{"version", 0, 0, 'v'},
 	{0, 0, 0, 0}
 };
@@ -80,10 +79,8 @@ static void closeAll(void)
 {
 	cout << "Exiting..." << endl;
 
-	// Close all indexes we may have opened
-	XapianDatabaseFactory::closeAll();
-
-	// Close the tokenizer libraries
+	// Close everything
+	ModuleFactory::unloadModules();
 	Dijon::FilterFactory::unloadFilters();
 	Dijon::HtmlFilter::shutdown();
 
@@ -135,6 +132,57 @@ static DBusHandlerResult filterHandler(DBusConnection *pConnection, DBusMessage
 		cout << "filterHandler: received NameOwnerChanged" << endl;
 #endif
 	}
+	// The first two signals are specified by the freedesktop.org Power Management spec v0.1 and v0.2
+	else if ((dbus_message_is_signal(pMessage, "org.freedesktop.PowerManagement", "BatteryStateChanged") == TRUE) ||
+		(dbus_message_is_signal(pMessage, "org.freedesktop.PowerManagement", "OnBatteryChanged") == TRUE) ||
+		(dbus_message_is_signal(pMessage, "org.gnome.PowerManager", "OnAcChanged") == TRUE))
+	{
+		DBusError error;
+		gboolean onBattery = FALSE;
+
+#ifdef DEBUG
+		cout << "filterHandler: received OnBatteryChanged" << endl;
+#endif
+		dbus_error_init(&error);
+		if ((dbus_message_get_args(pMessage, &error,
+			DBUS_TYPE_BOOLEAN, &onBattery,
+			DBUS_TYPE_INVALID) == TRUE) &&
+			(pData != NULL))
+		{
+			DaemonState *pServer = (DaemonState *)pData;
+
+			if (dbus_message_is_signal(pMessage, "org.gnome.PowerManager", "OnAcChanged") == TRUE)
+			{
+				// This tells us if we are on AC, not on battery
+				if (onBattery == TRUE)
+				{
+					onBattery = FALSE;
+				}
+				else
+				{
+					onBattery = TRUE;
+				}
+			}
+
+			if (onBattery == TRUE)
+			{
+				// We are now on battery
+				pServer->set_flag(DaemonState::ON_BATTERY);
+				pServer->stop_crawling();
+
+				cout << "System is now on battery" << endl;
+			}
+			else
+			{
+				// Back on-line
+				pServer->reset_flag(DaemonState::ON_BATTERY);
+				pServer->start_crawling();
+
+				cout << "System is now on AC" << endl;
+			}
+		}
+		dbus_error_free(&error);
+	}
 
 	return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 }
@@ -167,17 +215,58 @@ static DBusHandlerResult messageHandler(DBusConnection *pConnection, DBusMessage
 	return DBUS_HANDLER_RESULT_HANDLED;
 }
 
+static bool getBatteryState(DBusGConnection *pBus, const string &name, const string &path,
+	const string &method, gboolean &result)
+{
+	bool callSuccess = true;
+
+	if ((pBus == NULL) ||
+		(name.empty() == true) ||
+		(path.empty() == true) ||
+		(method.empty() == true))
+	{
+		return false;
+	}
+
+	DBusGProxy *pBusProxy = dbus_g_proxy_new_for_name(pBus, name.c_str(),
+		path.c_str(), name.c_str());
+	if (pBusProxy == NULL)
+	{
+		return false;
+	}
+
+	GError *pError = NULL;
+	if (dbus_g_proxy_call(pBusProxy, method.c_str(), &pError,
+		G_TYPE_INVALID,
+		G_TYPE_BOOLEAN, &result,
+		G_TYPE_INVALID) == FALSE)
+	{
+		if (pError != NULL)
+		{
+			cerr << "Couldn't get battery state: " << pError->message << endl;
+			g_error_free(pError);
+		}
+
+		callSuccess = false;
+	}
+
+	g_object_unref(pBusProxy);
+
+	return callSuccess;
+}
+
 int main(int argc, char **argv)
 {
 	string prefixDir(PREFIX);
 	struct sigaction newAction;
 	int longOptionIndex = 0, priority = 15;
-	bool overwriteIndex = false;
-	bool upgradeIndex = false;
+	bool resetHistory = false;
+	bool resetLabels = false;
 	bool fullScan = false;
+	bool reindex = false;
 
 	// Look at the options
-	int optionChar = getopt_long(argc, argv, "fhp:v", g_longOptions, &longOptionIndex);
+	int optionChar = getopt_long(argc, argv, "fhp:rv", g_longOptions, &longOptionIndex);
 	while (optionChar != -1)
 	{
 		switch (optionChar)
@@ -190,9 +279,10 @@ int main(int argc, char **argv)
 				cout << "pinot-dbus-daemon - D-Bus search and index daemon\n\n"
 					<< "Usage: pinot-dbus-daemon [OPTIONS]\n\n"
 					<< "Options:\n"
-					<< "  -f, --fullscan		force a full scan\n"
+					<< "  -f, --fullscan	force a full scan\n"
 					<< "  -h, --help		display this help and exit\n"
 					<< "  -p, --priority	set the daemon's priority (default 15)\n"
+					<< "  -r, --reindex		force a reindex\n"
 					<< "  -v, --version		output version information and exit\n"
 					<< "\nReport bugs to " << PACKAGE_BUGREPORT << endl;
 				return EXIT_SUCCESS;
@@ -207,6 +297,9 @@ int main(int argc, char **argv)
 					}
 				}
 				break;
+			case 'r':
+				reindex = true;
+				break;
 			case 'v':
 				cout << "pinot-dbus-daemon - " << PACKAGE_STRING << "\n\n" 
 					<< "This is free software.  You may redistribute copies of it under the terms of\n"
@@ -218,7 +311,7 @@ int main(int argc, char **argv)
 		}
 
 		// Next option
-		optionChar = getopt_long(argc, argv, "fhp:v", g_longOptions, &longOptionIndex);
+		optionChar = getopt_long(argc, argv, "fhp:rv", g_longOptions, &longOptionIndex);
 	}
 
 #if defined(ENABLE_NLS)
@@ -234,13 +327,11 @@ int main(int argc, char **argv)
 	}
 	// Initialize the GType and the D-Bus thread system
 	g_type_init();
-#if DBUS_VERSION > 1000000
+#if DBUS_NUM_VERSION > 1000000
 	dbus_threads_init_default();
 #endif
 	dbus_g_thread_init();
 
-	MIMEScanner::initialize();
-	DownloaderInterface::initialize();
 	g_refMainLoop = Glib::MainLoop::create();
 	Glib::set_application_name("Pinot DBus Daemon");
 
@@ -297,6 +388,21 @@ int main(int argc, char **argv)
 		pidFile.close();
 	}
 
+	// Initialize utility classes
+	if (MIMEScanner::initialize(PinotSettings::getHomeDirectory() + "/.local",
+		string(SHARED_MIME_INFO_PREFIX)) == false)
+	{
+		cerr << "Couldn't load MIME settings" << endl;
+	}
+	DownloaderInterface::initialize();
+	// Load tokenizer libraries, if any
+	Dijon::HtmlFilter::initialize();
+	Dijon::FilterFactory::loadFilters(string(LIBDIR) + "/pinot/filters");
+	Dijon::FilterFactory::loadFilters(confDirectory + "/filters");
+	// Load backends, if any
+	ModuleFactory::loadModules(string(LIBDIR) + "/pinot/backends");
+	ModuleFactory::loadModules(confDirectory + "/backends");
+
 	// Localize language names
 	Languages::setIntlName(0, _("Unknown"));
 	Languages::setIntlName(1, _("Danish"));
@@ -315,10 +421,6 @@ int main(int argc, char **argv)
 	Languages::setIntlName(14, _("Swedish"));
 	Languages::setIntlName(15, _("Turkish"));
 
-	// Load tokenizer libraries, if any
-	Dijon::HtmlFilter::initialize();
-	Dijon::FilterFactory::loadFilters(string(LIBDIR) + "/pinot/filters");
-	Dijon::FilterFactory::loadFilters(confDirectory + "/filters");
 	// Load the settings
 	settings.loadGlobal(string(SYSCONFDIR) + "/pinot/globalconfig.xml");
 	settings.load();
@@ -332,14 +434,16 @@ int main(int argc, char **argv)
 	sigaction(SIGQUIT, &newAction, NULL);
 
 	// Open the daemon index in read-write mode 
-	XapianDatabase *pDb = XapianDatabaseFactory::getDatabase(settings.m_daemonIndexLocation, false);
-	if ((pDb == NULL) ||
-		(pDb->isOpen() == false))
+	bool wasObsoleteFormat = false;
+	if (ModuleFactory::openOrCreateIndex(settings.m_defaultBackend, settings.m_daemonIndexLocation, wasObsoleteFormat, false) == false)
 	{
 		cerr << "Couldn't open index " << settings.m_daemonIndexLocation << endl;
 		return EXIT_FAILURE;
 	}
-	upgradeIndex = pDb->wasObsoleteFormat();
+	if (wasObsoleteFormat == true)
+	{
+		resetHistory = resetLabels = true;
+	}
 
 	// Do the same for the history database
 	PinotSettings::checkHistoryDatabase();
@@ -377,26 +481,6 @@ int main(int argc, char **argv)
 	else cout << "Set priority to " << priority << endl;
 #endif
 
-	// What version of the daemon is this ?
-	double daemonVersion = atof(VERSION);
-	if (daemonVersion > 0.0)
-	{
-		// What version is the index at ?
-		XapianIndex daemonIndex(settings.m_daemonIndexLocation);
-		double indexVersion = daemonIndex.getVersion();
-
-		// Is an upgrade necessary ?
-		if ((indexVersion < PINOT_INDEX_MIN_VERSION) &&
-			(daemonIndex.getDocumentsCount() > 0))
-		{
-			cout << "Upgrading index from version " << indexVersion << " to " << daemonVersion << endl;
-
-			// Yes, it is
-			overwriteIndex = upgradeIndex = true;
-		}
-		daemonIndex.setVersion(daemonVersion);
-	}
-
 	GError *pError = NULL;
 	DBusGConnection *pBus = dbus_g_bus_get(DBUS_BUS_SESSION, &pError);
 	if (pBus == NULL)
@@ -404,6 +488,10 @@ int main(int argc, char **argv)
 		if (pError != NULL)
 		{
 			cerr << "Couldn't open bus connection: " << pError->message << endl;
+			if (pError->message != NULL)
+			{
+				cerr << "Error is " << pError->message << endl;
+			}
 			g_error_free(pError);
 		}
 
@@ -424,31 +512,63 @@ int main(int argc, char **argv)
 	dbus_connection_set_exit_on_disconnect(pConnection, FALSE);
 	dbus_connection_setup_with_g_main(pConnection, NULL);
 
-	if ((dbus_error_is_set(&error) == FALSE) &&
-		(dbus_connection_register_object_path(pConnection, g_pinotDBusObjectPath,
-			&g_callVTable, &server) == TRUE))
+	if (dbus_connection_register_object_path(pConnection, g_pinotDBusObjectPath,
+			&g_callVTable, &server) == TRUE)
 	{
 		// Request to be identified by this name
 		// FIXME: flags are currently broken ?
 		dbus_bus_request_name(pConnection, g_pinotDBusService, 0, &error);
+		if (dbus_error_is_set(&error) == FALSE)
+		{
+			// See power management signals
+			dbus_bus_add_match(pConnection,
+				"type='signal',interface='org.freedesktop.PowerManagement'", &error);
+			dbus_bus_add_match(pConnection,
+				"type='signal',interface='org.gnome.PowerManager'", &error);
 
-		dbus_connection_add_filter(pConnection, (DBusHandleMessageFunction)filterHandler, &server, NULL);
+			dbus_connection_add_filter(pConnection,
+				(DBusHandleMessageFunction)filterHandler, &server, NULL);
+		}
+		else
+		{
+			cerr << "Couldn't obtain name " << g_pinotDBusService << endl;
+			if (error.message != NULL)
+			{
+				cerr << "Error is " << error.message << endl;
+			}
+		}
 
 		try
 		{
-			server.getQuitSignal().connect(SigC::slot(&quitAll));
+			set<string> labels;
+			bool gotLabels = false;
+			bool onBattery = false;
 
-			// Connect to threads' finished signal
-			server.connect();
-
-			if (overwriteIndex == true)
+			IndexInterface *pIndex = settings.getIndex(settings.m_daemonIndexLocation);
+			if (pIndex != NULL)
 			{
-				// Close and overwrite the index
-				XapianDatabaseFactory::closeAll();
-				XapianDatabaseFactory::getDatabase(settings.m_daemonIndexLocation, false, true);
+				string indexVersion(pIndex->getVersion());
+				gotLabels = pIndex->getLabels(labels);
+				// What version is the index at ?
+				if (indexVersion < PINOT_INDEX_MIN_VERSION)
+				{
+					cout << "Upgrading index from version " << indexVersion << " to " << VERSION << endl;
+
+					reindex = true;
+				}
+				if (reindex == true)
+				{
+					// Reset the index so that all documents are reindexed
+					pIndex->reset();
+
+					cout << "Reset index" << endl;
+
+					resetHistory = resetLabels = true;
+				}
+				pIndex->setVersion(VERSION);
 			}
 
-			if (upgradeIndex == true)
+			if (resetHistory == true)
 			{
 				CrawlHistory history(historyDatabase);
 				map<unsigned int, string> sources;
@@ -461,6 +581,66 @@ int main(int argc, char **argv)
 					history.deleteItems(sourceIter->first);
 					history.deleteSource(sourceIter->first);
 				}
+
+				cout << "Reset crawler history" << endl;
+			}
+
+			if ((resetLabels == true) &&
+				(pIndex != NULL))
+			{
+				// Re-apply the labels list
+				if (gotLabels == false)
+				{
+					// If this is an upgrade from a version < 0.80, the labels list
+					// needs to be pulled from the configuration file
+					pIndex->setLabels(settings.m_labels);
+
+					cout << "Set labels as per the configuration file" << endl;
+				}
+				else
+				{
+					pIndex->setLabels(labels);
+				}
+			}
+
+			if (pIndex != NULL)
+			{
+				delete pIndex;
+			}
+
+			// Connect to the quit signal
+			server.getQuitSignal().connect(sigc::ptr_fun(&quitAll));
+
+			// Connect to threads' finished signal
+			server.connect();
+
+			// Try and get the battery state
+			gboolean result = FALSE;
+			if ((getBatteryState(pBus, "org.freedesktop.PowerManagement",
+				"/org/freedesktop/PowerManagement", "GetOnBattery", result) == true) ||
+				(getBatteryState(pBus, "org.freedesktop.PowerManagement",
+				"/org/freedesktop/PowerManagement", "GetBatteryState", result) == true))
+			{
+				if (result == TRUE)
+				{
+					onBattery = true;
+				}
+			}
+			else if (getBatteryState(pBus, "org.gnome.PowerManager",
+				"/org/gnome/PowerManager", "GetOnAc", result) == true)
+			{
+				if (result == FALSE)
+				{
+					onBattery = true;
+				}
+			}
+			if (onBattery == true)
+			{
+				// We are on battery
+				server.set_flag(DaemonState::ON_BATTERY);
+				server.stop_crawling();
+
+				cout << "System is on battery" << endl;
 			}
 
 			server.start(fullScan);
@@ -487,7 +667,7 @@ int main(int argc, char **argv)
 	}
 	else
 	{
-		cerr << "Couldn't register object path: " << pError->message << endl;
+		cerr << "Couldn't register object path" << endl;
 	}
 	dbus_error_free(&error);
 
