@@ -29,9 +29,10 @@
 #include <glibmm/ustring.h>
 #include <glibmm/miscutils.h>
 #include <glibmm/convert.h>
+#include "config.h"
 extern "C"
 {
-#if DBUS_VERSION < 1000000
+#if DBUS_NUM_VERSION < 1000000
 #define DBUS_API_SUBJECT_TO_CHANGE
 #endif
 #include <dbus/dbus.h>
@@ -40,18 +41,15 @@ extern "C"
 }
 #include <gtkmm/main.h>
 
+#include "NLS.h"
 #include "FilterFactory.h"
 #include "Languages.h"
 #include "MIMEScanner.h"
-#include "XapianDatabase.h"
-#include "XapianDatabaseFactory.h"
+#include "ModuleFactory.h"
 #include "ActionQueue.h"
 #include "QueryHistory.h"
 #include "ViewHistory.h"
 #include "DownloaderInterface.h"
-#include "XapianIndex.h"
-#include "config.h"
-#include "NLS.h"
 #include "PinotSettings.h"
 #include "mainWindow.hh"
 
@@ -77,10 +75,8 @@ static void closeAll(void)
 		cerr << "Couldn't save configuration file" << endl;
 	}
 
-	// Close all indexes we may have opened
-	XapianDatabaseFactory::closeAll();
-
-	// Close the tokenizer libraries
+	// Close everything
+	ModuleFactory::unloadModules();
 	Dijon::FilterFactory::unloadFilters();
 	Dijon::HtmlFilter::shutdown();
 
@@ -112,7 +108,7 @@ int main(int argc, char **argv)
 	Glib::ustring errorMsg;
 	struct sigaction newAction;
 	int longOptionIndex = 0;
-	bool upgradeIndex = false;
+	bool warnAboutVersion = false;
 
 	// Look at the options
 	int optionChar = getopt_long(argc, argv, "hv", g_longOptions, &longOptionIndex);
@@ -156,13 +152,11 @@ int main(int argc, char **argv)
 	}
 	// Initialize the GType and the D-Bus thread system
 	g_type_init();
-#if DBUS_VERSION > 1000000
+#if DBUS_NUM_VERSION > 1000000
 	dbus_threads_init_default();
 #endif
 	dbus_g_thread_init();
 
-	MIMEScanner::initialize();
-	DownloaderInterface::initialize();
 	Gtk::Main m(&argc, &argv);
 	Glib::set_application_name("Pinot GTK2 UI");
 
@@ -223,6 +217,21 @@ int main(int argc, char **argv)
 		cerr.rdbuf(g_outputFile.rdbuf());
 	}
 
+	// Initialize utility classes
+	if (MIMEScanner::initialize(PinotSettings::getHomeDirectory() + "/.local",
+		string(SHARED_MIME_INFO_PREFIX)) == false)
+	{
+		cerr << "Couldn't load MIME settings" << endl;
+	}
+	DownloaderInterface::initialize();
+	// Load tokenizer libraries, if any
+	Dijon::HtmlFilter::initialize();
+	Dijon::FilterFactory::loadFilters(string(LIBDIR) + "/pinot/filters");
+	Dijon::FilterFactory::loadFilters(confDirectory + "/filters");
+	// Load backends, if any
+	ModuleFactory::loadModules(string(LIBDIR) + "/pinot/backends");
+	ModuleFactory::loadModules(confDirectory + "/backends");
+
 	// Localize language names
 	Languages::setIntlName(0, _("Unknown"));
 	Languages::setIntlName(1, _("Danish"));
@@ -244,10 +253,6 @@ int main(int argc, char **argv)
 	// Load search engines
 	settings.loadSearchEngines(prefixDir + "/share/pinot/engines");
 	settings.loadSearchEngines(confDirectory + "/engines");
-	// Load tokenizer libraries, if any
-	Dijon::HtmlFilter::initialize();
-	Dijon::FilterFactory::loadFilters(string(LIBDIR) + "/pinot/filters");
-	Dijon::FilterFactory::loadFilters(confDirectory + "/filters");
 	// Load the settings
 	settings.loadGlobal(string(SYSCONFDIR) + "/pinot/globalconfig.xml");
 	settings.load();
@@ -260,9 +265,8 @@ int main(int argc, char **argv)
 	sigaction(SIGQUIT, &newAction, NULL);
 
 	// Open this index read-write
-	XapianDatabase *pFirstDb = XapianDatabaseFactory::getDatabase(settings.m_docsIndexLocation, false);
-	if ((pFirstDb == NULL) ||
-		(pFirstDb->isOpen() == false))
+	bool wasObsoleteFormat = false;
+	if (ModuleFactory::openOrCreateIndex(settings.m_defaultBackend, settings.m_docsIndexLocation, wasObsoleteFormat, false) == false)
 	{
 		errorMsg = _("Couldn't open index");
 		errorMsg += " ";
@@ -270,13 +274,13 @@ int main(int argc, char **argv)
 	}
 	else
 	{
-		upgradeIndex = pFirstDb->wasObsoleteFormat();
+		warnAboutVersion = wasObsoleteFormat;
 	}
 	// ...and the daemon index in read-only mode
 	// If it can't be open, it just means the daemon has not yet created it
-	XapianDatabase *pSecondDb = XapianDatabaseFactory::getDatabase(settings.m_daemonIndexLocation);
+	ModuleFactory::openOrCreateIndex(settings.m_defaultBackend, settings.m_daemonIndexLocation, wasObsoleteFormat, true);
 	// Merge these two, this will be useful later
-	XapianDatabaseFactory::mergeDatabases("MERGED", pFirstDb, pSecondDb);
+	ModuleFactory::mergeIndexes(settings.m_defaultBackend, "MERGED", settings.m_docsIndexLocation, settings.m_daemonIndexLocation);
 
 	// Do the same for the history database
 	string historyDatabase(settings.getHistoryDatabaseName());
@@ -305,28 +309,28 @@ int main(int argc, char **argv)
 
 	atexit(closeAll);
 
-	// What version of the UI is this ?
-	double uiVersion = atof(VERSION);
-	if (uiVersion > 0)
+	IndexInterface *pIndex = settings.getIndex(settings.m_docsIndexLocation);
+	if (pIndex != NULL)
 	{
-		// What version is the index at ?
-		XapianIndex docsIndex(settings.m_docsIndexLocation);
-		double indexVersion = docsIndex.getVersion();
+		string indexVersion(pIndex->getVersion());
 
+		// What version is the index at ?
 		// Is an upgrade necessary ?
 		if ((indexVersion < PINOT_INDEX_MIN_VERSION) &&
-			(docsIndex.getDocumentsCount() > 0))
+			(pIndex->getDocumentsCount() > 0))
 		{
-			// Yes, it is
-			upgradeIndex = true;
+			warnAboutVersion = true;
 		}
-		docsIndex.setVersion(uiVersion);
-	}
+#ifdef DEBUG
+		cout << "My Web Pages was set to version " << indexVersion << endl;
+#endif
+		pIndex->setVersion(VERSION);
 
-	if ((upgradeIndex == true) &&
-		(errorMsg.empty() == true))
+		delete pIndex;
+	}
+	if (warnAboutVersion == true)
 	{
-		errorMsg = _("Updating all documents in My Web Pages is recommended");
+		settings.m_warnAboutVersion = warnAboutVersion;
 	}
 
 	try
@@ -338,7 +342,6 @@ int main(int argc, char **argv)
 		mainWindow mainBox;
 		if (errorMsg.empty() == false)
 		{
-			// FIXME: use a popup ! 
 			mainBox.set_status(errorMsg);
 		}
 		m.run(mainBox);
