@@ -18,15 +18,17 @@
 
 #include <stdlib.h>
 #include <iostream>
+#include <algorithm>
 #include <glibmm/convert.h>
 #include <gdkmm/color.h>
 #include <gtkmm/colorselection.h>
 #include <gtkmm/menu.h>
 #include <gtkmm/messagedialog.h>
 
-#include "SearchEngineFactory.h"
 #include "config.h"
 #include "NLS.h"
+#include "StringManip.h"
+#include "ModuleFactory.h"
 #include "PinotUtils.h"
 #include "prefsDialog.hh"
 
@@ -99,17 +101,9 @@ prefsDialog::prefsDialog() :
 	// Allow only single selection
 	patternsTreeview->get_selection()->set_mode(SELECTION_SINGLE);
 	populate_patternsTreeview();
-	if (m_settings.m_isBlackList == true)
-	{
-		patternsCombobox->set_active(0);
-	}
-	else
-	{
-		patternsCombobox->set_active(1);
-	}
 
 	// Hide the Google API entry field ?
-	if (SearchEngineFactory::isSupported("googleapi") == false)
+	if (ModuleFactory::isSupported("googleapi") == false)
 	{
 		apiKeyLabel->hide();
 		apiKeyEntry->hide();
@@ -128,6 +122,11 @@ prefsDialog::~prefsDialog()
 {
 }
 
+const set<string> &prefsDialog::getLabelsToAdd(void) const
+{
+	return m_addedLabels;
+}
+
 const set<string> &prefsDialog::getLabelsToDelete(void) const
 {
 	return m_deletedLabels;
@@ -154,8 +153,30 @@ void prefsDialog::populate_labelsTreeview()
 {
 	TreeModel::iterator iter;
 	TreeModel::Row row;
+	set<string> &labels = m_settings.m_labels;
+
+	// Get labels directly from the daemon's index
+	IndexInterface *pDaemonIndex = m_settings.getIndex(m_settings.m_daemonIndexLocation);
+	if (pDaemonIndex != NULL)
+	{
+		set<string> indexLabels;
+
+		// Nothing might be found if we are upgrading from an older version
+		// and the daemon has not been run
+		if (pDaemonIndex->getLabels(indexLabels) == true)
+		{
+			labels.clear();
+
+			copy(indexLabels.begin(), indexLabels.end(),
+				inserter(labels, labels.begin()));
+		}
+#ifdef DEBUG
+		else cout << "prefsDialog::populate_labelsTreeview: relying on configuration file" << endl;
+#endif
+
+		delete pDaemonIndex;
+	}
 
-	const set<string> &labels = m_settings.getLabels();
 	if (labels.empty() == true)
 	{
 		// This button will stay disabled until labels are added to the list
@@ -183,8 +204,9 @@ void prefsDialog::populate_labelsTreeview()
 
 void prefsDialog::save_labelsTreeview()
 {
-	// Clear the current settings
-	m_settings.clearLabels();
+	set<string> &labels = m_settings.m_labels;
+
+	labels.clear();
 
 	// Go through the labels tree
 	TreeModel::Children children = m_refLabelsTree->children();
@@ -196,8 +218,8 @@ void prefsDialog::save_labelsTreeview()
 			TreeModel::Row row = *iter;
 
 			// Add this new label to the settings
-			ustring labelName = row[m_labelsColumns.m_name];
-			ustring oldName = row[m_labelsColumns.m_oldName];
+			ustring labelName(row[m_labelsColumns.m_name]);
+			ustring oldName(row[m_labelsColumns.m_oldName]);
 			// Was this label renamed ?
 			if ((row[m_labelsColumns.m_enabled] == true) &&
 				(labelName != oldName))
@@ -211,11 +233,16 @@ void prefsDialog::save_labelsTreeview()
 			{
 				m_deletedLabels.erase(labelIter);
 			}
+			// Is this a nw label ?
+			if (row[m_labelsColumns.m_enabled] == false)
+			{
+				m_addedLabels.insert(from_utf8(labelName));
+			}
 
 #ifdef DEBUG
 			cout << "prefsDialog::save_labelsTreeview: " << labelName << endl;
 #endif
-			m_settings.addLabel(labelName);
+			labels.insert(labelName);
 		}
 	}
 }
@@ -224,6 +251,7 @@ void prefsDialog::populate_directoriesTreeview()
 {
 	TreeModel::iterator iter;
 	TreeModel::Row row;
+	ustring dirsString;
 
 	if (m_settings.m_indexableLocations.empty() == true)
 	{
@@ -242,14 +270,16 @@ void prefsDialog::populate_directoriesTreeview()
 		row = *iter;
 		row[m_directoriesColumns.m_monitor] = dirIter->m_monitor;
 		row[m_directoriesColumns.m_location] = dirIter->m_name;
+		dirsString += dirIter->m_name + (dirIter->m_monitor == true ? "1" : "0") + "|";
 	}
 
+	m_directoriesHash = StringManip::hashString(dirsString);
 	removeDirectoryButton->set_sensitive(true);
 }
 
 bool prefsDialog::save_directoriesTreeview()
 {
-	bool startService = false;
+	string dirsString;
 
 	// Clear the current settings
 	m_settings.m_indexableLocations.clear();
@@ -282,17 +312,26 @@ bool prefsDialog::save_directoriesTreeview()
 			cout << "prefsDialog::save_directoriesTreeview: " << indexableLocation.m_name << endl;
 #endif
 			m_settings.m_indexableLocations.insert(indexableLocation);
-			startService = true;
+			dirsString += indexableLocation.m_name + (indexableLocation.m_monitor == true ? "1" : "0") + "|";
 		}
 	}
 
-	return startService;
+	if (m_directoriesHash != StringManip::hashString(dirsString))
+	{
+#ifdef DEBUG
+		cout << "prefsDialog::save_directoriesTreeview: directories changed" << endl;
+#endif
+		return true;
+	}
+
+	return false;
 }
 
 void prefsDialog::populate_patternsTreeview()
 {
 	TreeModel::iterator iter;
 	TreeModel::Row row;
+	ustring patternsString;
 
 	if (m_settings.m_filePatternsList.empty() == true)
 	{
@@ -306,20 +345,39 @@ void prefsDialog::populate_patternsTreeview()
 		patternIter != m_settings.m_filePatternsList.end();
 		++patternIter)
 	{
+		ustring pattern(*patternIter);
+
 		// Create a new row
 		iter = m_refPatternsTree->append();
 		row = *iter;
 		// Set its name
-		row[m_patternsColumns.m_location] = *patternIter;
+		row[m_patternsColumns.m_location] = pattern;
+		patternsString += pattern + "|";
 	}
 	patternsCombobox->append_text(_("Exclude these patterns from indexing"));
 	patternsCombobox->append_text(_("Only index these patterns"));
 
 	removePatternButton->set_sensitive(true);
+
+	// Is it a black or white list ?
+	if (m_settings.m_isBlackList == true)
+	{
+		patternsCombobox->set_active(0);
+		patternsString += "0";
+	}
+	else
+	{
+		patternsCombobox->set_active(1);
+		patternsString += "1";
+	}
+
+	m_patternsHash = StringManip::hashString(patternsString);
 }
 
-void prefsDialog::save_patternsTreeview()
+bool prefsDialog::save_patternsTreeview()
 {
+	ustring patternsString;
+
 	// Clear the current settings
 	m_settings.m_filePatternsList.clear();
 
@@ -336,17 +394,30 @@ void prefsDialog::save_patternsTreeview()
 			if (pattern.empty() == false)
 			{
 				m_settings.m_filePatternsList.insert(pattern);
+				patternsString += pattern + "|";
 			}
 		}
 	}
 	if (patternsCombobox->get_active_row_number() == 0)
 	{
 		m_settings.m_isBlackList = true;
+		patternsString += "0";
 	}
 	else
 	{
 		m_settings.m_isBlackList = false;
+		patternsString += "1";
 	}
+
+	if (m_patternsHash != StringManip::hashString(patternsString))
+	{
+#ifdef DEBUG
+		cout << "prefsDialog::save_patternsTreeview: patterns changed" << endl;
+#endif
+		return true;
+	}
+
+	return false;
 }
 
 void prefsDialog::on_prefsOkbutton_clicked()
@@ -380,8 +451,9 @@ void prefsDialog::on_prefsOkbutton_clicked()
 	// Validate the current lists
 	save_labelsTreeview();
 	bool startForDirectories = save_directoriesTreeview();
-	save_patternsTreeview();
-	if (startForDirectories == true)
+	bool startForPatterns = save_patternsTreeview();
+	if ((startForDirectories == true) ||
+		(startForPatterns == true))
 	{
 		// Save the settings
 		m_settings.save();
@@ -420,9 +492,9 @@ void prefsDialog::on_removeLabelButton_clicked()
 		// Unselect
 		labelsTreeview->get_selection()->unselect(iter);
 		// Select another row
-		TreeModel::Path path = m_refLabelsTree->get_path(iter);
-		path.next();
-		labelsTreeview->get_selection()->select(path);
+		TreeModel::Path labelPath = m_refLabelsTree->get_path(iter);
+		labelPath.next();
+		labelsTreeview->get_selection()->select(labelPath);
 		// Erase
 		TreeModel::Row row = *iter;
 		m_deletedLabels.insert(from_utf8(row[m_labelsColumns.m_name]));
@@ -476,9 +548,9 @@ void prefsDialog::on_removeDirectoryButton_clicked()
 		// Unselect
 		directoriesTreeview->get_selection()->unselect(iter);
 		// Select another row
-		TreeModel::Path path = m_refDirectoriesTree->get_path(iter);
-		path.next();
-		directoriesTreeview->get_selection()->select(path);
+		TreeModel::Path dirPath = m_refDirectoriesTree->get_path(iter);
+		dirPath.next();
+		directoriesTreeview->get_selection()->select(dirPath);
 
 		// Erase
 		TreeModel::Row row = *iter;
@@ -528,8 +600,8 @@ void prefsDialog::on_addPatternButton_clicked()
 	TreeViewColumn *pColumn = patternsTreeview->get_column(0);
 	if (pColumn != NULL)
 	{
-		TreeModel::Path path = m_refPatternsTree->get_path(iter);
-		patternsTreeview->set_cursor(path, *pColumn, true);
+		TreeModel::Path patternPath = m_refPatternsTree->get_path(iter);
+		patternsTreeview->set_cursor(patternPath, *pColumn, true);
 	}
 
 	if (wasEmpty == true)
@@ -548,9 +620,9 @@ void prefsDialog::on_removePatternButton_clicked()
 		// Unselect
 		patternsTreeview->get_selection()->unselect(iter);
 		// Select another row
-		TreeModel::Path path = m_refPatternsTree->get_path(iter);
-		path.next();
-		patternsTreeview->get_selection()->select(path);
+		TreeModel::Path patternPath = m_refPatternsTree->get_path(iter);
+		patternPath.next();
+		patternsTreeview->get_selection()->select(patternPath);
 
 		// Erase
 		TreeModel::Row row = *iter;
