@@ -22,12 +22,11 @@
 #include <gtkmm/menu.h>
 
 #include "config.h"
-#include "Languages.h"
 #include "NLS.h"
+#include "Languages.h"
 #include "TimeConverter.h"
 #include "PinotUtils.h"
 #include "queryDialog.hh"
-#include "dateDialog.hh"
 
 using namespace std;
 using namespace Glib;
@@ -37,52 +36,25 @@ queryDialog::queryDialog(QueryProperties &properties) :
 	queryDialog_glade(),
 	m_name(properties.getName()),
 	m_properties(properties),
-	m_labels(PinotSettings::getInstance().getLabels()),
 	m_badName(true)
 {
-	unsigned int day, month, year;
-
 	// Name
-	string name(m_properties.getName());
-	if (name.empty() == true)
+	if (m_name.empty() == true)
 	{
 		queryOkbutton->set_sensitive(false);
 	}
 	else
 	{
-		nameEntry->set_text(to_utf8(name));
+		nameEntry->set_text(m_name);
 	}
 	// Query text
 	RefPtr<TextBuffer> refBuffer = queryTextview->get_buffer();
 	if (refBuffer)
 	{
-		refBuffer->set_text(to_utf8(m_properties.getFreeQuery()));
+		refBuffer->set_text(m_properties.getFreeQuery());
 	}
 	// Maximum number of results
 	resultsCountSpinbutton->set_value((double)m_properties.getMaximumResultsCount());
-	// Date range
-	bool enable = m_properties.getMinimumDate(day, month, year);
-	if (year > 0)
-	{
-		m_fromDate.set_dmy((Date::Day )day, (Date::Month )month, (Date::Year )year);
-	}
-	else
-	{
-		m_fromDate.set_time(time(NULL));
-	}
-	fromCheckbutton->set_active(enable);
-	fromButton->set_label(m_fromDate.format_string("%x"));
-	enable = m_properties.getMaximumDate(day, month, year);
-	if (year > 0)
-	{
-		m_toDate.set_dmy((Date::Day )day, (Date::Month )month, (Date::Year )year);
-	}
-	else
-	{
-		m_toDate.set_time(time(NULL));
-	}
-	toCheckbutton->set_active(enable);
-	toButton->set_label(m_toDate.format_string("%x"));
 	// Index all results
 	indexCheckbutton->set_active(m_properties.getIndexResults());
 
@@ -94,15 +66,82 @@ queryDialog::~queryDialog()
 {
 }
 
+bool queryDialog::is_separator(const RefPtr<TreeModel>& model, const TreeModel::iterator& iter)
+{
+	if (iter)
+	{
+		const TreeModel::Path queryPath = model->get_path(iter);
+		string rowPath(from_utf8(queryPath.to_string()));
+		unsigned int rowPos = 0;
+
+		// FIXME: this is extremely hacky !
+		if ((sscanf(rowPath.c_str(), "%u", &rowPos) == 1) &&
+			(rowPos == 10))
+		{
+			return true;
+		}
+	}
+
+	return false;
+}
+
 void queryDialog::populate_comboboxes()
 {
 	unsigned int labelNum = 1;
 
+	// All supported filters
+	filterCombobox->set_row_separator_func(sigc::mem_fun(*this, &queryDialog::is_separator));
+	filterCombobox->append_text(_("Host name"));
+	filterCombobox->append_text(_("File name"));
+	filterCombobox->append_text(_("File extension"));
+	filterCombobox->append_text(_("Title"));
+	filterCombobox->append_text(_("URL"));
+	filterCombobox->append_text(_("Directory"));
+	filterCombobox->append_text(_("Language code"));
+	filterCombobox->append_text(_("MIME type"));
+	filterCombobox->append_text(_("MIME class"));
+	filterCombobox->append_text(_("Label"));
+	// And separate numerical ranges
+	filterCombobox->append_text("===");
+	filterCombobox->append_text(_("Date"));
+	filterCombobox->append_text(_("Time"));
+	filterCombobox->append_text(_("Size"));
+	filterCombobox->set_active(0);
+
+	// Sort order
+	sortOrderCombobox->append_text(_("By relevance"));
+	sortOrderCombobox->append_text(_("By date"));
+	if (m_properties.getSortOrder() == QueryProperties::DATE)
+	{
+		sortOrderCombobox->set_active(1);
+	}
+	else
+	{
+		sortOrderCombobox->set_active(0);
+	}
+
+	// Stemming language
+	stemmingCombobox->append_text(_("None"));
+	stemmingCombobox->set_active(0);
+	string language(m_properties.getStemmingLanguage());
+	for (unsigned int languageNum = 1; languageNum < Languages::m_count; ++languageNum)
+	{
+		string languageName(Languages::getIntlName(languageNum));
+
+		stemmingCombobox->append_text(to_utf8(languageName));
+		// Is this the language we are looking for ?
+		if (language == languageName)
+		{
+			stemmingCombobox->set_active(languageNum);
+		}
+	}
+
+	// Labels
 	labelNameCombobox->append_text(_("None"));
 	labelNameCombobox->set_active(0);
-
 	// Add all labels to the label combo and select the one defined for the query
-	for (set<string>::const_iterator labelIter = m_labels.begin(); labelIter != m_labels.end(); ++labelIter)
+	set<string> &labels = PinotSettings::getInstance().m_labels;
+	for (set<string>::const_iterator labelIter = labels.begin(); labelIter != labels.end(); ++labelIter)
 	{
 		string labelName(*labelIter);
 
@@ -114,18 +153,6 @@ void queryDialog::populate_comboboxes()
 
 		++labelNum;
 	}
-
-	// All supported filters
-	filterCombobox->append_text(_("Host name"));
-	filterCombobox->append_text(_("File name"));
-	filterCombobox->append_text(_("File extension"));
-	filterCombobox->append_text(_("Title"));
-	filterCombobox->append_text(_("URL"));
-	filterCombobox->append_text(_("Directory"));
-	filterCombobox->append_text(_("Language code"));
-	filterCombobox->append_text(_("MIME type"));
-	filterCombobox->append_text(_("Label"));
-	filterCombobox->set_active(0);
 }
 
 bool queryDialog::badName(void) const
@@ -135,16 +162,18 @@ bool queryDialog::badName(void) const
 
 void queryDialog::on_queryOkbutton_clicked()
 {
+	ustring newName(nameEntry->get_text());
+
 	// Name
-	m_properties.setName(from_utf8(nameEntry->get_text()));
+	m_properties.setName(newName);
 	m_badName = false;
 	// Did the name change ?
-	if (m_name != m_properties.getName())
+	if (m_name != newName)
 	{
 		const std::map<string, QueryProperties> &queries = PinotSettings::getInstance().getQueries();
 
 		// Is it already used ?
-		std::map<string, QueryProperties>::const_iterator queryIter = queries.find(m_properties.getName());
+		std::map<string, QueryProperties>::const_iterator queryIter = queries.find(newName);
 		if (queryIter != queries.end())
 		{
 			// Yes, it is
@@ -159,13 +188,27 @@ void queryDialog::on_queryOkbutton_clicked()
 	RefPtr<TextBuffer> refBuffer = queryTextview->get_buffer();
 	if (refBuffer)
 	{
-		m_properties.setFreeQuery(from_utf8(refBuffer->get_text()));
+		m_properties.setFreeQuery(refBuffer->get_text());
 	}
 	// Maximum number of results
 	m_properties.setMaximumResultsCount((unsigned int)resultsCountSpinbutton->get_value());
-	// Date range
-	m_properties.setMinimumDate(fromCheckbutton->get_active(), m_fromDate.get_day(), m_fromDate.get_month(), m_fromDate.get_year());
-	m_properties.setMaximumDate(toCheckbutton->get_active(), m_toDate.get_day(), m_toDate.get_month(), m_toDate.get_year());
+	// Sort order
+	if (sortOrderCombobox->get_active_row_number() == 1)
+	{
+		m_properties.setSortOrder(QueryProperties::DATE);
+	}
+	else
+	{
+		m_properties.setSortOrder(QueryProperties::RELEVANCE);
+	}
+	// Stemming language
+	string languageName(from_utf8(stemmingCombobox->get_active_text()));
+	int chosenLanguagePos = stemmingCombobox->get_active_row_number();
+	if (chosenLanguagePos == 0)
+	{
+		languageName.clear();
+	}
+	m_properties.setStemmingLanguage(languageName);
 	// Index all results
 	m_properties.setIndexResults(indexCheckbutton->get_active());
 	// Index label
@@ -193,6 +236,8 @@ void queryDialog::on_nameEntry_changed()
 void queryDialog::on_addFilterButton_clicked()
 {
 	ustring filter;
+	time_t timeT = time(NULL);
+	struct tm *tm = localtime(&timeT);
 
 	// What's the corresponding filter ?
 	int chosenFilter = filterCombobox->get_active_row_number();
@@ -200,31 +245,48 @@ void queryDialog::on_addFilterButton_clicked()
 	switch (chosenFilter)
 	{
 		case 0:
-			filter = "site";
+			filter = "site:localhost";
 			break;
 		case 1:
-			filter = "file";
+			filter = "file:xxx.txt";
 			break;
 		case 2:
-			filter = "ext";
+			filter = "ext:txt";
 			break;
 		case 3:
-			filter = "title";
+			filter = "title:pinot";
 			break;
 		case 4:
-			filter = "url";
+			filter = "url:file:///home/xxx/yyy.txt";
 			break;
 		case 5:
-			filter = "dir";
+			filter = "dir:/home/xxx";
 			break;
 		case 6:
-			filter = "lang";
+			filter = "lang:en";
 			break;
 		case 7:
-			filter = "type";
+			filter = "type:text/plain";
 			break;
 		case 8:
-			filter = "label";
+			filter = "class:text";
+			break;
+		case 9:
+			filter = "label:New";
+			break;
+		case 10:
+			// Separator
+			break;
+		case 11:
+			filter = TimeConverter::toYYYYMMDDString(tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday);
+			filter += "..20991231";
+			break;
+		case 12:
+			filter = TimeConverter::toHHMMSSString(tm->tm_hour, tm->tm_min, tm->tm_sec);
+			filter += "..235959";
+			break;
+		case 13:
+			filter += "0..10240b";
 			break;
 		default:
 			break;
@@ -236,38 +298,7 @@ void queryDialog::on_addFilterButton_clicked()
 		ustring queryText = refBuffer->get_text();
 		queryText += " ";
 		queryText += filter;
-		queryText += ":xxx";
 		refBuffer->set_text(queryText);
 	}
 }
 
-void queryDialog::on_fromButton_clicked()
-{
-	dateDialog datePicker(_("Start date"), m_fromDate);
-
-#ifdef DEBUG
-	cout << "queryDialog::on_fromButton_clicked: start date is " << m_fromDate.format_string("%x") << endl;
-#endif
-	datePicker.show();
-	if (datePicker.run() == RESPONSE_OK)
-	{
-		datePicker.getChoice(m_fromDate);
-		fromButton->set_label(m_fromDate.format_string("%x"));
-	}
-}
-
-void queryDialog::on_toButton_clicked()
-{
-	dateDialog datePicker(_("End date"), m_toDate);
-
-#ifdef DEBUG
-	cout << "queryDialog::on_toButton_clicked: end date is " << m_toDate.format_string("%x") << endl;
-#endif
-	datePicker.show();
-	if (datePicker.run() == RESPONSE_OK)
-	{
-		datePicker.getChoice(m_toDate);
-	        toButton->set_label(m_toDate.format_string("%x"));
-	}
-}
-
