@@ -21,7 +21,10 @@
 
 #include <pthread.h>
 #include <string>
+#include <list>
 #include <map>
+#include <set>
+#include <vector>
 
 #include "Url.h"
 
@@ -38,8 +41,11 @@ class MIMEAction
 		MIMEAction(const MIMEAction &other);
 		virtual ~MIMEAction();
 
+		bool operator<(const MIMEAction &other) const;
+
 		MIMEAction &operator=(const MIMEAction &other);
 
+		void load(void);
 		void parseExec(void);
 
 		bool m_multipleArgs;
@@ -52,6 +58,34 @@ class MIMEAction
 
 };
 
+/** Caching of MIME type information.
+  * A cache can be reloaded as required.
+  */
+class MIMECache
+{
+	public:
+		MIMECache();
+		MIMECache(const std::string &file, const std::string &section);
+		MIMECache(const MIMECache& other);
+		~MIMECache();
+
+		bool operator<(const MIMECache &other) const;
+
+		MIMECache &operator=(const MIMECache &other);
+
+		bool load(const std::list<std::string> &desktopFilesPaths);
+		void reload(const std::list<std::string> &desktopFilesPaths);
+  
+		std::string m_file;
+		std::string m_section;
+		std::multimap<std::string, MIMEAction> m_defaultActions;
+
+	protected:
+		bool findDesktopFile(const std::string &desktopFile, const std::string &mimeType,
+			std::map<std::string, MIMEAction> &loadedActions);
+
+};
+
 /**
   * Utility class to get a file's MIME type and the default application associated with it.
   */
@@ -61,30 +95,46 @@ class MIMEScanner
 		~MIMEScanner();
 
 		/// Initializes the MIME system.
-		static void initialize(void);
-
+		static bool initialize(const std::string &userPrefix, const std::string &systemPrefix);
+  
 		/// Shutdowns the MIME system.
 		static void shutdown(void);
 
+		/// Lists MIME configuration files under the given prefix.
+		static void listConfigurationFiles(const std::string &prefix, std::set<std::string> &files);
+
 		/// Finds out the given file's MIME type.
 		static std::string scanFile(const std::string &fileName);
 
 		/// Finds out the given URL's MIME type.
 		static std::string scanUrl(const Url &urlObj);
 
+		/// Gets parent MIME types.
+		static bool getParentTypes(const std::string &mimeType, std::set<std::string> &parentMimeTypes);
+
 		/// Adds a user-defined action for the given type.
 		static void addDefaultAction(const std::string &mimeType, const MIMEAction &typeAction);
 
-		/// Determines the default action for the given type.
-		static bool getDefaultAction(const std::string &mimeType, MIMEAction &typeAction);
+		/// Determines the default action(s) for the given type.
+		static bool getDefaultActions(const std::string &mimeType, std::vector<MIMEAction> &typeActions);
 
 	protected:
-		static pthread_mutex_t m_mutex;
-		static std::map<std::string, MIMEAction> m_defaultActions;
+		/// Mutex to protect access to xdgmime.
+		static pthread_mutex_t m_xdgMutex;
+		/// Lock to protect access to caches.
+		static pthread_rwlock_t m_cachesLock;
+		/// MIME type caches, ordered by decreasing priority.
+		static std::list<MIMECache> m_caches;
 
 		MIMEScanner();
 
 		static std::string scanFileType(const std::string &fileName);
+  
+		static bool addCache(const std::string &file, const std::string &section,
+			const std::list<std::string> &desktopFilesPaths);
+  
+		static bool getDefaultActionsForType(const std::string &mimeType, std::set<std::string> &actionNames,
+			std::vector<MIMEAction> &typeActions);
 
 	private:
 		MIMEScanner(const MIMEScanner &other);
